﻿/*
 * Declaration for the "ExpandFolder" pseudo-namespace for opening and closing the TOC tree
 */
var ExpandFolder = function() {};

if((typeof jQuery) != 'function')
{
  alert(LocaleString.lcStrMessageNeedJQuery);
  ExpandFolder.rootURL = '';
}
else
{
  // Get the relative path to the standard folder from the path specification.
  ExpandFolder.rootURL = (($('script[src$="' + TemplateSettings.templateLibFileName + '"]').attr('src')) ? $('script[src$="' + TemplateSettings.templateLibFileName + '"]').attr('src').replace((TemplateSettings.javaScriptPath + TemplateSettings.templateLibFileName), '') : '');
}

// Configure display icons.
ExpandFolder.openIconPath  = ExpandFolder.rootURL + TemplateSettings.openIconPath;
ExpandFolder.closeIconPath = ExpandFolder.rootURL + TemplateSettings.closeIconPath;
ExpandFolder.fileIconPath  = ExpandFolder.rootURL + TemplateSettings.fileIconPath;

// Define styles.
ExpandFolder.styleOpenDiv       = TemplateSettings.styleOpenTreeItemDiv;
ExpandFolder.styleCloseDiv      = TemplateSettings.styleCloseTreeItemDiv;
ExpandFolder.styleLineIconDiv   = TemplateSettings.styleIconLineDiv;
ExpandFolder.styleLineNoIconDiv = TemplateSettings.styleNoIconLineDiv;
ExpandFolder.styleOpenImg       = TemplateSettings.styleOpenTreeItemImg;
ExpandFolder.styleCloseImg      = TemplateSettings.styleCloseTreeItemImg;
ExpandFolder.styleFileImg       = TemplateSettings.styleFileTreeItemImg;

// Attribute to configure the DIV tag open/close state.
ExpandFolder.attributeFolded = TemplateSettings.foldedTreeItemAttrId;
// Define jQuery selector.
ExpandFolder.selectorTreePaneDiv  = 'div#' + TemplateSettings.divTreePaneId;
ExpandFolder.selectorOpenItemDiv  = ExpandFolder.selectorTreePaneDiv + ' div[class=' + TemplateSettings.styleOpenTreeItemDiv + ']';
ExpandFolder.selectorCloseItemDiv = ExpandFolder.selectorTreePaneDiv + ' div[class=' + TemplateSettings.styleCloseTreeItemDiv + ']';
ExpandFolder.selectorOpenItemImg  = ExpandFolder.selectorTreePaneDiv + ' img[class=' + TemplateSettings.styleOpenTreeItemImg + ']';
ExpandFolder.selectorCloseItemImg = ExpandFolder.selectorTreePaneDiv + ' img[class=' + TemplateSettings.styleCloseTreeItemImg + ']';
ExpandFolder.selectorFileItemImg  = ExpandFolder.selectorTreePaneDiv + ' img[class=' + TemplateSettings.styleFileTreeItemImg + ']';
// Prefix of the ID specified when expanding or collapsing.
ExpandFolder.expandDivIdPrefix  = TemplateSettings.expandDivIdPrefix;
ExpandFolder.expandImgIdPrefix  = TemplateSettings.expandImgIdPrefix;

// Initial display.
ExpandFolder.initExpandFolder = function()
{
  $(ExpandFolder.selectorOpenItemDiv).each(
    function() { $(this).css('display', 'block').attr(ExpandFolder.attributeFolded, false); }
  );
  $(ExpandFolder.selectorCloseItemDiv).each(
    function() { $(this).css('display', 'none').attr(ExpandFolder.attributeFolded, true); }
  );
  $(ExpandFolder.selectorOpenItemImg).each(
    function() { $(this).attr('src', ExpandFolder.openIconPath); }
  );
  $(ExpandFolder.selectorCloseItemImg).each(
    function() { $(this).attr('src', ExpandFolder.closeIconPath); }
  );
  $(ExpandFolder.selectorFileItemImg).each(
    function() { $(this).attr('src', ExpandFolder.fileIconPath); }
  );
}

// Open all TOC folders.
ExpandFolder.openAll = function()
{
  $(ExpandFolder.selectorOpenItemDiv + ',' + ExpandFolder.selectorCloseItemDiv).each(
    function() { $(this).css('display', 'block').attr(ExpandFolder.attributeFolded, false); }
  );
  $(ExpandFolder.selectorCloseItemImg).each(
    function() { $(this).attr('src', ExpandFolder.openIconPath).attr('class', ExpandFolder.styleOpenImg); }
  );
}

// Close all TOC folders.
ExpandFolder.closeAll = function()
{
  $(ExpandFolder.selectorOpenItemDiv + ',' + ExpandFolder.selectorCloseItemDiv).each(
    function() { $(this).css('display', 'none').attr(ExpandFolder.attributeFolded, true); }
  );
  $(ExpandFolder.selectorOpenItemImg).each(
    function() { $(this).attr('src', ExpandFolder.closeIconPath).attr('class', ExpandFolder.styleCloseImg); }
  );
}

// Function called by the IMG tag on open/close.
// id       : ID of the DIV tag that opens or closes.
// objCaller: Caller IMG tag (generally passing "this")
ExpandFolder.expand = function(id, objCaller)
{
  $('#' + id).each(
    function()
    {
      var isObjDisplay = ($(this).css('display') == 'block');
      $(this).css('display', (isObjDisplay ? 'none' : 'block')).attr(ExpandFolder.attributeFolded, isObjDisplay);
      $(objCaller).attr('src', (isObjDisplay ? ExpandFolder.closeIconPath : ExpandFolder.openIconPath))
                  .attr('class', (isObjDisplay ? ExpandFolder.styleCloseImg : ExpandFolder.styleOpenImg));
    }
  );
}

