﻿/*
 * Declaration for the Reassemble pseudo-namespace for automatic numbering and table of contents (TOC) auto-generation features.
 */
var Reassemble = function() {};

// Table of contents settings.
// Size-related.
Reassemble.autoindexContentWidth    = TemplateSettings.autoindexContentWidth;
Reassemble.autoindexToggleWidth     = TemplateSettings.autoindexToggleWidth;
Reassemble.globalindexContentWidth  = TemplateSettings.globalindexContentWidth;
Reassemble.globalindexToggleWidth   = TemplateSettings.globalindexToggleWidth;
// ID stuff.
Reassemble.settingAttrId            = TemplateSettings.settingAttrId;
Reassemble.autoindexContentId       = TemplateSettings.autoindexContentId;
Reassemble.autoindexWrapperId       = TemplateSettings.autoindexWrapperId;
Reassemble.autoindexTabId           = TemplateSettings.autoindexTabId;
Reassemble.autoindexToggleId        = TemplateSettings.autoindexToggleId;
Reassemble.autoindexToggleAttrId    = TemplateSettings.autoindexToggleAttrId;
Reassemble.globalindexWrapperId     = TemplateSettings.globalindexWrapperId;
Reassemble.globalindexTabId         = TemplateSettings.globalindexTabId;
Reassemble.globalindexToggleId      = TemplateSettings.globalindexToggleId;
Reassemble.globalindexToggleAttrId  = TemplateSettings.globalindexToggleAttrId;
Reassemble.globalindexRootPaneId    = TemplateSettings.globalindexRootPaneId;
// Stylesheet stuff.
Reassemble.styleAutoindexContent    = TemplateSettings.styleAutoindexContent;
Reassemble.styleAutoindexWrapper    = TemplateSettings.styleAutoindexWrapper;
Reassemble.styleAutoindexTab        = TemplateSettings.styleAutoindexTab;
Reassemble.styleAutoindexToggle     = TemplateSettings.styleAutoindexToggle;
// Define jQuery selector.
Reassemble.selectorAutoindexContentDiv  = 'div#' + Reassemble.autoindexContentId;
Reassemble.selectorAutoindexWrapperDiv  = 'div#' + Reassemble.autoindexWrapperId;
Reassemble.selectorAutoindexTabDiv      = 'div#' + Reassemble.autoindexTabId;
Reassemble.selectorAutoindexToggleDiv   = 'div#' + Reassemble.autoindexToggleId;
Reassemble.selectorGlobalindexWrapperDiv= 'div#' + Reassemble.globalindexWrapperId;
Reassemble.selectorGlobalindexTabDiv    = 'div#' + Reassemble.globalindexTabId;
Reassemble.selectorGlobalindexToggleDiv = 'div#' + Reassemble.globalindexToggleId;

// Variables.
// Table of contents data.
Reassemble.objectTocData = null;
// Flag for the TOC tab.
Reassemble.autoindexEnabled = false;
Reassemble.globalindexEnabled = false;
// Object operated on by a scroll handler.
Reassemble.autoindexScrollTarget = null;
Reassemble.globalindexScrollTarget = null;
// Display mode (for guidelines).
Reassemble.viewMode = '0';

Reassemble.str2int = function(str)
{
  if (str == null) return 0;
  var val = parseInt(str);
  if (isNaN(val)) val = 0;
  return val;
}

// Add handler.
Reassemble.setEventHandler = function(eventname, func)
{
  if ((eventname == null) || (eventname == '')) return;
  if (func == null) return;
  if (window.attachEvent) window.attachEvent('on' + eventname, func);
  else if (window.addEventListener) window.addEventListener(eventname, func, false);
}

// Resize handler.
Reassemble.resizeHandler = function()
{
  if ((Reassemble.autoindexEnabled) || (Reassemble.globalindexEnabled))
  {
    var clientHeight = $(window).height();
    var clientWidth  = $(window).width();
    // alert("clientHeight=" + clientHeight + " clientWidth=" + clientWidth);
    if (clientWidth < 400) return;
    // Initial value for each size.
    var contentWidth = clientWidth;
    var autoindexWidth = 0;
    var globalindexWidth = Reassemble.globalindexContentWidth + Reassemble.globalindexToggleWidth;
    // Auto-generated TOC side.
    var attrAutoindexToggle = $(Reassemble.selectorAutoindexWrapperDiv).attr(Reassemble.autoindexToggleAttrId);
    if (Reassemble.autoindexEnabled)
    {
      if ((attrAutoindexToggle != null) && (attrAutoindexToggle == '0'))
      {
        $(Reassemble.selectorAutoindexTabDiv).css('display', 'none');
        autoindexWidth = Reassemble.autoindexToggleWidth;
      }
      else
      {
        $(Reassemble.selectorAutoindexTabDiv).css('display', 'block');
        autoindexWidth = Reassemble.autoindexContentWidth + Reassemble.autoindexToggleWidth;
      }
    }
    // Overall TOC side.
    var attrGlobalindexToggle = $(Reassemble.selectorGlobalindexWrapperDiv).attr(Reassemble.globalindexToggleAttrId);
    if ((attrGlobalindexToggle != null) && (attrGlobalindexToggle == '0'))
    {
      $(Reassemble.selectorGlobalindexTabDiv).css('display', 'none');
      globalindexWidth = Reassemble.globalindexToggleWidth;
    }
    else
    {
      $(Reassemble.selectorGlobalindexTabDiv).css('display', 'block');
      globalindexWidth = Reassemble.globalindexContentWidth + Reassemble.globalindexToggleWidth;
      // Adjust height of the TOC tree.
      if ($(ExpandFolder.selectorTreePaneDiv).offset() != null)
      {
        var treeOffsetTop = $(ExpandFolder.selectorTreePaneDiv).offset().top - $(window).scrollTop();
        if (clientHeight > (treeOffsetTop + 200))
        {
          $(ExpandFolder.selectorTreePaneDiv).height(clientHeight - treeOffsetTop - TemplateSettings.globalindexPanePadH);
        }
      }
    }
    contentWidth = clientWidth - autoindexWidth - globalindexWidth - TemplateSettings.autoindexContentPadW;
    // Apply size.
    $(Reassemble.selectorAutoindexContentDiv).width(contentWidth);
    $(Reassemble.selectorAutoindexContentDiv).css('margin-left', globalindexWidth);
    $(Reassemble.selectorAutoindexContentDiv).css('margin-right', autoindexWidth);
    $(Reassemble.selectorGlobalindexTabDiv).width(Reassemble.globalindexContentWidth);
    $(Reassemble.selectorGlobalindexTabDiv).css('margin-right', -Reassemble.globalindexContentWidth);
    $(Reassemble.selectorAutoindexWrapperDiv).width(autoindexWidth);
    $(Reassemble.selectorAutoindexWrapperDiv).css('left', clientWidth - autoindexWidth);
    $(Reassemble.selectorGlobalindexWrapperDiv).width(globalindexWidth);
    // API.
    $('iframe#' + TemplateSettings.apiIframeId).each(
      function ()
      {
        $(this).width(contentWidth - TemplateSettings.apiIframePadW);
        $(this).height(clientHeight - TemplateSettings.apiIframePadH);
      }
    );
    Reassemble.scrollHandler();
  }
}

// Scroll bar handler.
Reassemble.scrollHandler = function()
{
  if (Reassemble.autoindexEnabled && (Reassemble.autoindexScrollTarget != null))
  {
    $(Reassemble.autoindexScrollTarget).css('top', $(window).scrollTop());
    $(Reassemble.autoindexScrollTarget).css('left', $(window).width() - $(Reassemble.selectorAutoindexWrapperDiv).width() + $(window).scrollLeft());
  }
  if (Reassemble.globalindexEnabled && (Reassemble.globalindexScrollTarget != null))
  {
    $(Reassemble.globalindexScrollTarget).css('top', $(window).scrollTop());
    $(Reassemble.globalindexScrollTarget).css('left', $(window).scrollLeft());
  }
  if (Reassemble.autoindexEnabled) Reassemble.autoindexFocusIndexInvoke();
}

// Toggle-tab click handler (auto-generated TOC).
Reassemble.autoindexToggleHandler = function()
{
  if (Reassemble.autoindexEnabled)
  {
    var attrToggle = $(Reassemble.selectorAutoindexWrapperDiv).attr(Reassemble.autoindexToggleAttrId);
    $(Reassemble.selectorAutoindexWrapperDiv).attr(Reassemble.autoindexToggleAttrId, (((attrToggle != null) && (attrToggle == '0')) ? '1' : '0'));
    Reassemble.resizeHandler();
  }
}

// Toggle-tab click handler (overall TOC).
Reassemble.globalindexToggleHandler = function()
{
  if (Reassemble.globalindexEnabled)
  {
    var attrToggle = $(Reassemble.selectorGlobalindexWrapperDiv).attr(Reassemble.globalindexToggleAttrId);
    $(Reassemble.selectorGlobalindexWrapperDiv).attr(Reassemble.globalindexToggleAttrId, (((attrToggle != null) && (attrToggle == '0')) ? '1' : '0'));
    Reassemble.resizeHandler();
  }
}

/* Move focus of the auto-generated TOC. */
/* Repurposing code that is used in NW4C 1.1.0. */
Reassemble.autoindexFocusIndexInvoked=false;
Reassemble.selectorAutoindexFocusIndex = Reassemble.selectorAutoindexTabDiv + ' a[href*=#]';
Reassemble.autoindexFocusIndex = function()
{
  var a,b,e,d=$(window).scrollTop()+TemplateSettings.autoindexFocusOffset;
  $(Reassemble.selectorAutoindexFocusIndex).each(function(){var c='#'+$(this).attr('href').split('#')[1];if($(c).length&&($(this).parent().css('display')!='none')&&(e=$(c).offset().top,!(d<e)&&(!a||b<=e)))a=this,b=e});
  $(Reassemble.selectorAutoindexFocusIndex).each(function(){Reassemble.autoindexToggleClass($(this),'focused',this==a)});
  Reassemble.autoindexFocusIndexInvoked=false;
}
Reassemble.autoindexFocusIndexInvoke = function(){Reassemble.autoindexFocusIndexInvoked||(setTimeout(Reassemble.autoindexFocusIndex,100),Reassemble.autoindexFocusIndexInvoked=true)}
Reassemble.autoindexToggleClass = function(a,b,d){d?a.addClass(b):a.removeClass(b)}

//function reassemble()
Reassemble.reassemble = function()
{
  // Parameters passed by GET.
  var levels = '1';
  var figures = '';
  var tables = '';
  var codes = '';
  var forceNoIndex = false;
  var url = '';
  var q_search = '';

  // Control parameters.
  var startLevel = 1;
  var endLevel = 6;
  var offsetLevel = 0;
  var autoindex = false;
  var forceDefault = false;
  var forceNolabel = false;
  var forceNoChapter = false;
  var chaseIndexURL = true;
  var chaseIndexJavaScript = true;
  var folded = false;
  var noReset = false;

  // Add control parameters to the BODY tag.
  var attrBody = document.body.getAttribute(Reassemble.settingAttrId);
  if (attrBody != null)
  {
    var params = attrBody.split(',');
    for (var i = 0; i < params.length; i++)
    {
      var param = params[i];
      var ind = param.indexOf('=');
      if (ind < 0) continue;
      var key = decodeURIComponent(param.substring(0, ind));
      key = key.replace(/\s/g, '');
      var data = decodeURIComponent(param.slice(ind + 1));
      if (data == null) data = '';
      data = data.replace(/\s/g, '');
      if (key.match(/^start$/i))
      {
        startLevel = Reassemble.str2int(data);
      }
      else if (key.match(/^end$/i))
      {
        endLevel = Reassemble.str2int(data);
      }
      else if (key.match(/^autoindex$/i))
      {
        if (data.match(/^yes$|^true$/i)) autoindex = true;
        else
        if (data.match(/^no$|^false$/i)) autoindex = false;
      }
      else if (key.match(/^forcedefault$/i))
      {
        if (data.match(/^yes$|^true$/i)) forceDefault = true;
        else
        if (data.match(/^no$|^false$/i)) forceDefault = false;
      }
      else if (key.match(/^forcenolabel$/i))
      {
        if (data.match(/^yes$|^true$/i)) forceNolabel = true;
        else
        if (data.match(/^no$|^false$/i)) forceNolabel = false;
      }
      else if (key.match(/^forcenochapter$/i))
      {
        if (data.match(/^yes$|^true$/i)) forceNoChapter = true;
        else
        if (data.match(/^no$|^false$/i)) forceNoChapter = false;
      }
      else if (key.match(/^level$/i))
      {
        levels = data.replace(/\./g, ',');
      }
      else if (key.match(/^figure$/i))
      {
        figures = Reassemble.str2int(data);
      }
      else if (key.match(/^table$/i))
      {
        tables = Reassemble.str2int(data);
      }
      else if (key.match(/^code$/i))
      {
        codes = Reassemble.str2int(data);
      }
      else if (key.match(/^chaseindex$/i))
      {
        if (data.match(/^yes$/i))   { chaseIndexURL = true;     chaseIndexJavaScript = true; }
        else
        if (data.match(/^no$/i))    { chaseIndexURL = false;    chaseIndexJavaScript = false; }
        else
        if (data.match(/^url$/i))   { chaseIndexURL = true;     chaseIndexJavaScript = false; }
        else
        if (data.match(/^js$/i))    { chaseIndexURL = false;    chaseIndexJavaScript = true; }
      }
      else if (key.match(/^offset$/i))
      {
        offsetLevel = Reassemble.str2int(data);
      }
      else if (key.match(/^autoindextab$/i))
      {
        if (data.match(/^fold$/i)) folded = true;
        else
        if (data.match(/^open$/i)) folded = false;
      }
      else if (key.match(/^noreset$/i))
      {
        if (data.match(/^yes$|^true$/i)) noReset = true;
        else
        if (data.match(/^no$|^false$/i)) noReset = false;
      }
      else if (key.match(/^forcenoindex$/i))
      {
        if (data.match(/^yes$|^true$/i)) forceNoIndex = true;
        else
        if (data.match(/^no$|^false$/i)) forceNoIndex = false;
      }
    }
    if (startLevel < 1) startLevel = 1;
    if (startLevel > 6) startLevel = 6;
    if (endLevel   > 6) endLevel = 6;
    if (startLevel > endLevel) endLevel = startLevel;
    if (offsetLevel < -5) offsetLevel = -5;
    if (offsetLevel > 5) offsetLevel = 5;
  }

  // Parse parameters passed by GET.
  var query = location.search;
  query = query.slice(query.lastIndexOf('?')+1);
  var params = query.split('&');
  for (var i = 0; i < params.length; i++)
  {
    var param = params[i];
    var ind = param.indexOf('=');
    if (ind < 0) continue;
    var key = param.substring(0, ind);
    key = decodeURIComponent(key);
    key = key.replace(/\s/g, '');
    var data = decodeURIComponent(param.slice(ind + 1));
    if (data == null) data = '';
    data = data.replace(/\s/g, '');
    if (key.match(/^level$/i))
    {
      if (!forceDefault) levels = data;
    }
    else if (key.match(/^figure$/i))
    {
      if (!forceDefault) figures = data;
    }
    else if (key.match(/^table$/i))
    {
      if (!forceDefault) tables = data;
    }
    else if (key.match(/^code$/i))
    {
      if (!forceDefault) codes = data;
    }
    else if (key.match(/^noindex$/i))
    {
      if (data.match(/^yes$|^true$/i)) forceNoIndex = true;
      else
      if (data.match(/^no$|^false$/i)) forceNoIndex = false;
    }
    else if (key.match(/^url$/i))
    {
      url = data;
    }
    else if (key.match(/^highlighttext$/i))
    {
      q_search = param.slice(ind + 1);
      if (q_search == null) q_search = '';
      var q_array = q_search.split('+');
      q_search = '';
      for (var n = 0; n < q_array.length; n++) q_search += (((n > 0) ? ' ' : '') + decodeURIComponent(q_array[n]));
      //alert('q_search:' + q_search);
    }
  }

  // Display parameters.
  var arrayLevel = new Array(6);
  var arrayKeep  = new Array(6);
  var figure = Reassemble.str2int(figures); figure = (figure > 0) ? (figure - 1) : 0;
  var table  = Reassemble.str2int(tables);  table  = (table  > 0) ? (table  - 1) : 0;
  var code   = Reassemble.str2int(codes);   code   = (code   > 0) ? (code   - 1) : 0;

  levels = levels.split(',');
  for (var i = 0; i < 6; i++)
  {
    var n = (levels.length > i) ? Reassemble.str2int(levels[i]) : 0;
    n = (n > 0) ? (n - 1) : 0;
    arrayLevel[i] = n;
    arrayKeep[i]  = (levels.length > i) ? true : false;
  }

  // Prepare to auto-generate the TOC.
  var baseLevel = 7;
  var baseLevelNone = 7;
  var arrayIndex = new Array();
  arrayIndex.length = 0;

  // Auto-numbering.
  var elems;
  if (document.all)
  {
    // IE
    elems = document.all;
  }
  else if (document.getElementsByTagName)
  {
    // W3C
    elems = document.getElementsByTagName('*');
  }
  else
  {
    return;
  }
  if (elems.length)
  {
    for (var i = 0; i < elems.length; i++)
    {
      var elem = elems[i];
      var tagName = elem.tagName;
      var className = elem.className;
      var noLabel = false;
      var noChapter = false;
      var noIndex = false;
      var keep = false;
      var elemAttr = elem.getAttribute(Reassemble.settingAttrId);
      if (elemAttr != null)
      {
        var attrs = elemAttr.split(',');
        for (var x = 0; x < attrs.length; x++)
        {
          attrs[x] = attrs[x].replace(/\s/g, '');
          if (attrs[x].match(/^none$/i)) { noLabel = true; noIndex = true; }
          if (attrs[x].match(/^keep$/i)) keep = true;
          if (attrs[x].match(/^nochapter$/i)) noChapter = true;
          if (attrs[x].match(/^nolabel$/i)) noLabel = true;
          if (attrs[x].match(/^noindex$/i)) noIndex = true;
        }
      }
      if (forceNolabel) noLabel = true;
      if (forceNoChapter) noChapter = true;
      if (tagName.match(/^H[1-6]$/i))
      {
        if (!noLabel)
        {
          var current = Reassemble.str2int(tagName.slice(1)) - startLevel + offsetLevel;
          if ((current >= 0) && (current <= (endLevel - startLevel)))
          {
            for (var x = 0; x < current; x++)
            {
              if (arrayKeep[x]) arrayLevel[x] = Reassemble.str2int(levels[x]);
            }
            if (!keep)
            {
              for (var x = (current + 1); x < 6; x++)
              {
                arrayLevel[x] = 0; arrayKeep[x]  = false;
              }
              if ((current == 0) && !noReset)
              {
                // Reset numbers for figures, tables, and code blocks.
                figure = 0; table = 0; code = 0;
              }
            }
            arrayLevel[current]++; arrayKeep[current] = false;
            var str = '';
            if (current < 4)
            {
              for (var x = 0; x <= current; x++) str += (arrayLevel[x] + '.');
              elem.innerHTML = (noChapter ? '' : (str + ' ')) + elem.innerHTML;
            }
          }
        }
        if (autoindex && !noIndex)
        {
          var current = Reassemble.str2int(tagName.slice(1)) - startLevel;
          if ((current >= 0) && (current <= (endLevel - startLevel)))
          {
            var elemId = elem.id;
            if ((elemId == null) || (elemId == ''))
            {
              elemId = 'autoindex_' + arrayIndex.length;
              elem.id = elemId;
            }
            var data = current + '\t' + elemId + '\t' + elem.innerHTML;
            arrayIndex.push(data);
            if (!noLabel)
            {
              if (baseLevel > current) baseLevel = current;
            }
            else
            {
              if (baseLevelNone > current) baseLevelNone = current;
            }
          }
        }
      }
      else if (tagName.match(/^DIV$/i))
      {
        if (!noLabel && className.match(/^figure$/i))
        {
          figure++;
          elem.innerHTML = LocaleString.lcStrLabelFigure + (noChapter ? '' : (arrayLevel[0] + '-')) + figure + '. ' + elem.innerHTML;
        }
        else if (!noLabel && className.match(/^table$/i))
        {
          table++;
          elem.innerHTML = LocaleString.lcStrLabelTable + (noChapter ? '' : (arrayLevel[0] + '-')) + table + '. ' + elem.innerHTML;
        }
        else if (!noLabel && className.match(/^code$/i))
        {
          code++;
          elem.innerHTML = LocaleString.lcStrLabelCode + (noChapter ? '' : (arrayLevel[0] + '-')) + code + '. ' + elem.innerHTML;
        }
      }
    }
  }
  // Auto-generate the TOC.
  if(!forceNoIndex && autoindex && ((typeof jQuery) == 'function'))
  {
    Reassemble.autoindexEnabled = true;
    // Auto-generate the required DIV tags.
    // You must surround the text with `<div id="autoindex_content"></div>` when the script writes by using `document.write.`
    var objDivContent = document.getElementById(Reassemble.autoindexContentId);
    if (objDivContent == null)
    {
      $('body').wrapInner('<div id="' + Reassemble.autoindexContentId + '"></div>');
    }
    // Account for the possibility that the DIV tag was written by using the old template.
    var objDivWrapper = document.getElementById(Reassemble.autoindexWrapperId);
    if (objDivWrapper == null)
    {
      $('body').prepend('<div id="' + Reassemble.autoindexWrapperId + '"></div>');
    }
    var objDivIndex   = document.getElementById(Reassemble.autoindexTabId);
    if (objDivIndex == null)
    {
      $(Reassemble.selectorAutoindexWrapperDiv).prepend('<div id="' + Reassemble.autoindexTabId + '"></div>');
    }
    var objDivToggle  = document.getElementById(Reassemble.autoindexToggleId);
    if (objDivToggle == null)
    {
      $(Reassemble.selectorAutoindexWrapperDiv).prepend('<div id="' + Reassemble.autoindexToggleId + '"></div>');
    }
    if (baseLevel > 6) baseLevel = baseLevelNone;
    var prev = 0;
    var strHtml = '<ul>';
    // Link back to the top of the page.
    strHtml += ('<li><a href="#' + Reassemble.autoindexContentId + '" class="toplevel">' + LocaleString.lcStrLabelAcTopLevel + '</a></li>');
    for (var i = 0; i < arrayIndex.length; i++)
    {
      var inds = arrayIndex[i].split('\t');
      if (inds.length > 2)
      {
        var level = Reassemble.str2int(inds[0]);
        var id = inds[1];
        var text = inds[2];
        level -= baseLevel;
        if (level < 0) level = 0;
        if (level < prev)
        {
          for (var x = prev; x > level; x--) strHtml += '</ul>';
        }
        else if (level > prev)
        {
          for (var x = prev; x < level; x++) strHtml += '<ul>';
        }
        strHtml += ('<li><a href="#' + id + '" class="level' + level + '">' + text + '</a></li>');
        prev = level;
      }
    }
    for (var x = prev; x > 0; x--) strHtml += '</ul>';
    // Attribute setting.
    $(Reassemble.selectorAutoindexContentDiv).attr('class', Reassemble.styleAutoindexContent);
    $(Reassemble.selectorAutoindexTabDiv).attr('class', Reassemble.styleAutoindexTab).html(strHtml);
    $(Reassemble.selectorAutoindexToggleDiv).attr('class', Reassemble.styleAutoindexToggle).click(Reassemble.autoindexToggleHandler);
    $(Reassemble.selectorAutoindexWrapperDiv).attr(Reassemble.autoindexToggleAttrId, (folded ? '0' : '1'));
    // Auto Padding
    var pads = ['15px', '30px', '40px', '50px', '60px', '70px', '70px', '70px', '70px', '70px'];
    $(Reassemble.selectorAutoindexTabDiv + ' a').each(
      function(){if(this.className.match(/^level[0-9]/i)){var a=parseInt(this.className.slice(5));var b='0px 0px 0px '+pads[(isNaN(a)?'0px':a)];this.style.padding=b;}}
    );
    // DIV tag that is operated on by the scroll handler.
    if ($(Reassemble.selectorAutoindexWrapperDiv).css('position') != 'fixed') Reassemble.autoindexScrollTarget = document.getElementById(Reassemble.autoindexWrapperId);
  }

  if(!forceNoIndex && ((typeof jQuery) == 'function') && ($(TemplateSettings.selectorTocFile).attr('src') != null) && (Reassemble.objectTocData == null))
  {
    alert(LocaleString.lcStrMessageNothingToc);
  }

  // Create the overall TOC when tocData.js is enabled.
  // You must surround the text with `<div id="autoindex_content"></div>` when the script writes by using `document.write.`
  if(!forceNoIndex && ((typeof jQuery) == 'function') && ($(TemplateSettings.selectorTocFile).attr('src') != null) && (Reassemble.objectTocData != null))
  {
    Reassemble.globalindexEnabled = true;

    // Auto-generate the required DIV tags.
    var objDivContent = document.getElementById(Reassemble.autoindexContentId);
    if (objDivContent == null)
    {
      $('body').wrapInner('<div id="' + Reassemble.autoindexContentId + '"></div>');
    }
    $(Reassemble.selectorAutoindexContentDiv).attr('class', Reassemble.styleAutoindexContent);
    $('body').prepend('<div id="' + Reassemble.globalindexWrapperId + '"><div id="' + Reassemble.globalindexToggleId + '"></div><div id="' + Reassemble.globalindexTabId + '"></div></div>');

    $(Reassemble.selectorGlobalindexTabDiv).prepend('<div id="' + Reassemble.globalindexRootPaneId + '"></div>');
    $('div#' + Reassemble.globalindexRootPaneId)
      .append('<div id="' + TemplateSettings.titleLogoDivId + '" class="' + TemplateSettings.styleTitleLogoDiv + '" align="center"></div>')
      .append('<div id="' + TemplateSettings.titleTocDivId + '" class="' + TemplateSettings.styleTitleTocDiv + '" align="center"></div>')
      .append('<div id="' + TemplateSettings.searchBoxMiniDivId + '" align="center"></div>')
      .append('<div id="' + TemplateSettings.printControllerId + '" align="left"></div>')
      .append('<div id="' + TemplateSettings.treeControllerId + '"></div>')
      .append('<div id="' + TemplateSettings.divTreePaneId + '" class="' + TemplateSettings.styleTreePane + '"></div>');
    $('div#' + TemplateSettings.printControllerId)
      .append('<input type="radio" id="' + TemplateSettings.changeViewModeIdPrefix + '0" name="mode" value="0" onClick="Reassemble.changeViewMode(\'0\');" checked />' + LocaleString.lcStrLabelModeAll  + '<br />')
      .append('<input type="radio" id="' + TemplateSettings.changeViewModeIdPrefix + '1" name="mode" value="1" onClick="Reassemble.changeViewMode(\'1\');" />' + LocaleString.lcStrLabelModeDesc + '<br />')
      .append('<input type="radio" id="' + TemplateSettings.changeViewModeIdPrefix + '2" name="mode" value="2" onClick="Reassemble.changeViewMode(\'2\');" />' + LocaleString.lcStrLabelModeItem + '<br />')
      .append('<br />');
    $('div#' + TemplateSettings.treeControllerId)
      .append('<input type="button" value="OpenAll" onClick="ExpandFolder.openAll();" />&nbsp;')
      .append('<input type="button" value="CloseAll" onClick="ExpandFolder.closeAll();" />');
    $(Reassemble.selectorGlobalindexToggleDiv).click(Reassemble.globalindexToggleHandler);
    $(Reassemble.selectorGlobalindexWrapperDiv).attr(Reassemble.globalindexToggleAttrId, '1');
    // Overall TOC.
    if (Reassemble.objectTocData != null)
    {
      var rootURL = $(TemplateSettings.selectorTocFile).attr('src').replace(TemplateSettings.templateTocFileName, '');
      Reassemble.objectTocData.setRootURL(rootURL);
      Reassemble.objectTocData.createContents();
      ExpandFolder.initExpandFolder();
    }
    // Move the tree focus.
    if (chaseIndexURL || chaseIndexJavaScript)
    {
      if (chaseIndexURL) Reassemble.focusIndexURL(location.href);
//      if (chaseIndexJavaScript && (url != '')) Reassemble.focusIndexJS(url);
    }
    // DIV tag that is operated on by the scroll handler.
    if ($(Reassemble.selectorGlobalindexWrapperDiv).css('position') != 'fixed') Reassemble.globalindexScrollTarget = document.getElementById(Reassemble.globalindexWrapperId);
  }
  // Search highlighting.
  if ((q_search != '') && ((typeof jQuery) == 'function'))
  {
    var r_array = Search.parseQuery(q_search);
    var strReg = '';
    for (var x = 0; x < r_array.length; x++)
    {
      if (r_array[x] == '') continue;
      if (r_array[x].match(/^ +$/)) continue;
      if (r_array[x].charAt(0) == '\t') continue;
      var str = Search.escapeRegex(r_array[x]);
      if (strReg != '') strReg += '|';
      strReg += str;
    }
    if (strReg != '')
    {
      //alert('strReg:' + strReg);
      var objDivContent = document.getElementById(Reassemble.autoindexContentId);
      if (objDivContent != null)
      {
        $(document).ready(Reassemble.highlightText(document, objDivContent, strReg, 'search_highlight'));
      }
      else
      {
        $(document).ready(Reassemble.highlightText(document, document.body, strReg, 'search_highlight'));
      }
/*
      // The IFRAME is accessible, but nothing can be obtained if the nodeType is 3, so they can't be highlighted.
      var objIframe = document.getElementById(TemplateSettings.apiIframeId);
      if ((objIframe != null) && (objIframe.contentWindow != null))
      {
        $(objIframe.contentWindow.document).ready(Reassemble.highlightText(objIframe.contentWindow.document, objIframe.contentWindow.document.body, strReg, 'search_highlight'));
      }
*/
    }
  }
  // Reproduce the TOC state from the cookie.
  Reassemble.loadSetting();
  // Mode changes if guidelines are displayed.
  Reassemble.changeViewMode(Reassemble.viewMode);
  // Call resize and scroll, and adjust size.
  Reassemble.resizeHandler();
  Reassemble.scrollHandler();
}

/* Move the TOC focus. (This functionality updates the existing code from NW4C 1.1.0.) */
Reassemble.selectorGlobalindexFocusIndex = ExpandFolder.selectorTreePaneDiv + ' a';
Reassemble.focusIndexJS = function(url)
{
  if (url == null) return;
  url = url.replace(/\.\.\//g, '');
  var a=null;
  $(Reassemble.selectorGlobalindexFocusIndex).each(function(){var b=$(this).attr('href');var c=b.match(/viewPage\('([^,]+)',[^\)]+\)/i);var d=RegExp.$1.replace(/\.\.\//g,'');if(c!=null&&d!=null&&url.lastIndexOf(d)!=-1)a=b});
  $(Reassemble.selectorGlobalindexFocusIndex).each(function(){($(this).attr('href')==a)?$(this).addClass('focused'):$(this).removeClass('focused')});
}

Reassemble.focusIndexURL = function(url)
{
  if (url == null) return;
  var a=null;
  $(Reassemble.selectorGlobalindexFocusIndex).each(function(){var b=$(this).attr('href');if(url.lastIndexOf(b.replace(/\.\.\//g,''))!=-1)a=b});
  $(Reassemble.selectorGlobalindexFocusIndex).each(function(){($(this).attr('href')==a)?$(this).addClass('focused'):$(this).removeClass('focused')});
}

// Save the TOC state to the cookie.
Reassemble.setCookie = function (name, value, path)
{
  var str = '';
  str += (name + '=' + encodeURIComponent(value));
  if (path) str += ('; path=' + path);
  document.cookie = str;
}

Reassemble.saveSetting = function ()
{
  if (Reassemble.autoindexEnabled)
  {
    var attr = null;
    // Toggle.
    attr = $(Reassemble.selectorAutoindexWrapperDiv).attr(Reassemble.autoindexToggleAttrId);
    Reassemble.setCookie(TemplateSettings.cookieKeyAutoindexToggle, ((attr != null) ? attr : '1'), '/');
  }
  if ((Reassemble.globalindexEnabled) && (document.getElementById))
  {
    var attr = null;
    // Toggle.
    attr = $(Reassemble.selectorGlobalindexWrapperDiv).attr(Reassemble.globalindexToggleAttrId);
    Reassemble.setCookie(TemplateSettings.cookieKeyGlobalindexToggle, ((attr != null) ? attr : '1'), '/');
    // Tree.
    $(ExpandFolder.selectorTreePaneDiv + ' div[' + ExpandFolder.attributeFolded + ']').each(
      function () { Reassemble.setCookie($(this).attr('id'), $(this).attr(ExpandFolder.attributeFolded), '/'); }
    );
    // mode
    Reassemble.setCookie(TemplateSettings.cookieKeyViewmode, Reassemble.viewMode, '/');
  }
}

Reassemble.loadSetting = function ()
{
  //alert('loadSetting');
  var settings = document.cookie.split(';');
  for (var n = 0; n < settings.length; n++)
  {
    var data = settings[n].split('=');
    var key = data[0].replace(/^ */, '');
    var value = decodeURIComponent(data[1]);
    //alert ('"' + key + '"="' + value + '"');
    if (Reassemble.autoindexEnabled)
    {
      if (key == TemplateSettings.cookieKeyAutoindexToggle) $(Reassemble.selectorAutoindexWrapperDiv).attr(Reassemble.autoindexToggleAttrId, value);
    }
    if (Reassemble.globalindexEnabled)
    {
      if (key == TemplateSettings.cookieKeyGlobalindexToggle)
      {
        $(Reassemble.selectorGlobalindexWrapperDiv).attr(Reassemble.globalindexToggleAttrId, value);
      }
      else
      {
        var obj = document.getElementById(key);
        if (obj)
        {
          $(obj).css('display', (value == 'true' ? 'none' : 'block')).attr(ExpandFolder.attributeFolded, value);
          $('img#' + ExpandFolder.expandImgIdPrefix + key)
            .attr('src', (value == 'true' ? ExpandFolder.closeIconPath : ExpandFolder.openIconPath))
            .attr('class', (value == 'true' ? ExpandFolder.styleCloseImg : ExpandFolder.styleOpenImg));
        }
      }
    }
    if (key == TemplateSettings.cookieKeyViewmode)
    {
      var nMode = parseInt(value);
      if (isNaN(nMode)) nMode = 0;
      if ((nMode < 0) || (nMode > 2)) nMode = 0;
      Reassemble.viewMode = '' + nMode;
      // Apply to radio buttons.
      var obj = document.getElementById(TemplateSettings.changeViewModeIdPrefix + Reassemble.viewMode);
      if (obj) obj.checked = true;
    }
  }
}

// Change the guideline display mode.
Reassemble.selectorChangeViewMode = Reassemble.selectorAutoindexTabDiv + ' ul li a[href*=#' + TemplateSettings.guidelineItemLabelPrefix + ']';
Reassemble.changeViewMode = function (mode)
{
  var nMode = parseInt(mode);
  if (isNaN(nMode)) nMode = 0;
  if ((nMode < 0) || (nMode > 2)) nMode = 0;
  Reassemble.viewMode = '' + nMode;
  // If guidelines are displayed:
  //if ($('script[src$="viewPage.js"]').attr('src') != null)
  if ((typeof Guideline.initExpandGuideline) == 'function')
  {
    Guideline.initExpandGuideline();
    if (Reassemble.autoindexEnabled)
    {
      switch (nMode)
      {
        case 1:
          $(Reassemble.selectorChangeViewMode).parent().css('display', 'none');
          Reassemble.autoindexFocusIndexInvoke();
          break;
        case 0:
        case 2:
        default:
          $(Reassemble.selectorChangeViewMode).parent().css('display', 'block');
          Reassemble.autoindexFocusIndexInvoke();
          break;
      }
    }
  }
}

// Page for printing.
Reassemble.viewPrintPage = function ()
{
  var category = $('select#' + TemplateSettings.selectCategoryId + ' option:selected').val();
  if ((category != null) && (category != ''))
  {
    if (objTOC != null)
    {
      var rootURL = $(TemplateSettings.selectorTocFile).attr('src').replace(TemplateSettings.templateTocFileName, '');
      location.href = rootURL + 'printpage.html?category=' + encodeURIComponent(category);
    }
  }
}
// Show highlighting.
Reassemble.highlightText = function(objDocument, node, regex, className)
{
  if (node.nodeType == 3)
  {
    var val = node.nodeValue;
    var re = new RegExp(regex, 'i');
    var pos = val.search(re);
    var text = re.exec(val);
    //alert('val:' + val);
    if (pos >= 0 && !($(node.parentNode).hasClass(className)))
    {
      text = text[0];
      //alert('text:' + text);
      var span = objDocument.createElement("span");
      span.className = className;
      span.appendChild(objDocument.createTextNode(val.substr(pos, text.length)));
      var textNode = objDocument.createTextNode(val.substr(pos + text.length));
      node.parentNode.insertBefore(span,
        node.parentNode.insertBefore(textNode, node.nextSibling)
      );
      node.nodeValue = val.substr(0, pos);
      Reassemble.highlightText(objDocument, textNode, regex, className);
    }
  }
  else if (!($(node).is("button, select, textarea")))
  {
    //alert(node.nodeType);
    $(node.childNodes).each(function() {
      Reassemble.highlightText(objDocument, this, regex, className);
    });
  }
};

// * Search declaration for the pseudo namespace that includes the search functions.
Reassemble.setEventHandler('load', Reassemble.reassemble);
Reassemble.setEventHandler('unload', Reassemble.saveSetting);
Reassemble.setEventHandler('scroll', Reassemble.scrollHandler);
Reassemble.setEventHandler('resize', Reassemble.resizeHandler);

