/*---------------------------------------------------------------------------*
Project: VoiceChat library
File: vct.h

Copyright (C) 2011-2014 Nintendo Co., Ltd.  All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#ifndef VCT_VOICECHAT_H_
#define VCT_VOICECHAT_H_

/*---------------------------------------------------------------------------*
Include
 *---------------------------------------------------------------------------*/
#ifdef NN_VCT_TARGET_CAFE

#include <cafe.h>

#elif NN_VCT_TARGET_CTR

#include <nn.h>

#endif


/*---------------------------------------------------------------------------*
Extern "C"
 *---------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif  // ifdef __cplusplus


/*---------------------------------------------------------------------------*
Define
 *---------------------------------------------------------------------------*/
// --- Common error codes.
#define VCT_ERROR_NONE                  0
#define VCT_ERROR_BAD_PARAM            -1
#define VCT_ERROR_BAD_MODE             -2
#define VCT_ERROR_BAD_REQUEST          -3
#define VCT_ERROR_SEND_FAIL            -4
#define VCT_ERROR_EXHAUST_CLIENTS      -6

// --- Session and client limitations.
#define VCT_MAX_SESSION                 16
#define VCT_MAX_CONFERENCE_CLIENT       16
#define VCT_MAX_CLIENT                  16

#define VCT_MAX_AUDIO_STREAM            (VCT_MAX_CONFERENCE_CLIENT - 1)

// Magic Number in VCT
#define VCT_MAGIC_PACKET_HEADER         0x5f564354  //!< Magic number in the VCT packet header.

#define VCT_CODEC_DEFAULT                   VCT_CODEC_4BIT_ADPCM  //!< Default audio codec.
#define VCT_SPARE_CODEC_DEFAULT             VCT_CODEC_2BIT_ADPCM  //!< Default audio codec for spare packets.

#define VCT_AUDIO_FRAME_LENGTH_MIN          64  //!< Specifies the minimum audio frame length (in milliseconds).
#define VCT_AUDIO_FRAME_LENGTH_MAX          192 //!< Specifies the maximum audio frame length (in milliseconds).
#define VCT_AUDIO_FRAME_LENGTH_DEFAULT      144 //!< Specifies the default audio frame length (in milliseconds).

#define VCT_AUDIO_DATA_SIZE_PER_MILLI_SEC   16

#define VCT_AUDIO_DATA_SIZE_MIN             (VCT_AUDIO_FRAME_LENGTH_MIN * VCT_AUDIO_DATA_SIZE_PER_MILLI_SEC)
#define VCT_AUDIO_DATA_SIZE_MAX             (VCT_AUDIO_FRAME_LENGTH_MAX * VCT_AUDIO_DATA_SIZE_PER_MILLI_SEC)
#define VCT_AUDIO_DATA_SIZE_DEFAULT         (VCT_AUDIO_FRAME_LENGTH_DEFAULT * VCT_AUDIO_DATA_SIZE_PER_MILLI_SEC)

#define VCT_AUDIO_FRAME_RATE                8000 //!< Audio frame rate (8 kHz).

// --- Receive buffer size.
#define VCT_AUDIO_HEADER_SIZE                   12  //!< Header size for VCT audio data packets.
#define VCT_AUDIO_2_BIT_ADPCM_PACKET_SIZE_MAX   (VCT_AUDIO_HEADER_SIZE + 2 * VCT_AUDIO_FRAME_LENGTH_MAX + 4)
#define VCT_AUDIO_3_BIT_ADPCM_PACKET_SIZE_MAX   (VCT_AUDIO_HEADER_SIZE + 3 * VCT_AUDIO_FRAME_LENGTH_MAX + 4)
#define VCT_AUDIO_4_BIT_ADPCM_PACKET_SIZE_MAX   (VCT_AUDIO_HEADER_SIZE + 4 * VCT_AUDIO_FRAME_LENGTH_MAX + 4)

#define VCT_AUDIO_SPARE_PACKET_SIZE_MAX         VCT_AUDIO_2_BIT_ADPCM_PACKET_SIZE_MAX

#define VCT_AUDIO_PACKET_BUFFER_SIZE            (VCT_AUDIO_DATA_SIZE_MAX + VCT_AUDIO_SPARE_PACKET_SIZE_MAX)

#ifdef NN_VCT_TARGET_CAFE
#define VCT_AUDIO_BUFFER_PACKET_HEADER          64
#elif NN_VCT_TARGET_CTR
#define VCT_AUDIO_BUFFER_PACKET_HEADER          56
#endif
#define VCT_AUDIO_BUFFER_SIZE                   (VCT_AUDIO_PACKET_BUFFER_SIZE + VCT_AUDIO_BUFFER_PACKET_HEADER)

#define VCT_DEFAULT_AUDIO_BUFFER_COUNT          8
#define VCT_MIN_AUDIO_BUFFER_COUNT              4
#define VCT_MAX_AUDIO_BUFFER_COUNT              (24 * VCT_MAX_AUDIO_STREAM)

#define VCT_DEFAULT_AUDIO_BUFFER_SIZE           (VCT_AUDIO_BUFFER_SIZE           \
                                               * VCT_DEFAULT_AUDIO_BUFFER_COUNT  \
                                               * VCT_MAX_AUDIO_STREAM)

#ifdef NN_VCT_TARGET_CAFE
  #define VCT_VAD_DETECT_GAIN_UP_DEFAULT                          350
  #define VCT_VAD_LOST_GAIN_UP_DEFAULT                            280
#elif NN_VCT_TARGET_CTR
  #define VCT_VAD_DETECT_GAIN_UP_DEFAULT                          350
  #define VCT_VAD_LOST_GAIN_UP_DEFAULT                            280
#endif

#define VCT_VAD_INVALID_SCALE_THRESHOLD_DEFAULT                 (7 * 7)
#define VCT_VAD_CLAMP_GAIN_MAX                                  0x3FFFFFFF
#define VCT_VAD_RELEASE_TIME_DEFAULT                            5       // 144 (default frame length) * 5 = 720 ms

// --- Library configuration.
#define VCT_CONFIG_FLAG_NONE                    0x00
#define VCT_CONFIG_FLAG_VAD                     0x01  //!< VAD (Voice Activity Detection) feature.
#define VCT_CONFIG_FLAG_ECHO_CANCEL             0x02  //!< Echo cancellation.
#define VCT_CONFIG_FLAG_HPF                     0x04  //!< High-pass filter.
#define VCT_CONFIG_FLAG_SPARE_PACKET            0x08  //!< Spare packet.
#define VCT_CONFIG_FLAG_DROP_RECOVER            0x20  //!< Drop recovery.
#define VCT_CONFIG_FLAG_BPF                     0x40  //!< Biquad filter.
#define VCT_CONFIG_FLAG_ALL                     0xff

#ifdef NN_VCT_TARGET_CAFE
  #define VCT_CONFIG_FLAG_DEFAULT                 (VCT_CONFIG_FLAG_VAD | \
                                                   VCT_CONFIG_FLAG_SPARE_PACKET | \
                                                   VCT_CONFIG_FLAG_DROP_RECOVER)
#elif NN_VCT_TARGET_CTR
  #define VCT_CONFIG_FLAG_DEFAULT                 (VCT_CONFIG_FLAG_VAD | \
                                                   VCT_CONFIG_FLAG_ECHO_CANCEL | \
                                                   VCT_CONFIG_FLAG_SPARE_PACKET | \
                                                   VCT_CONFIG_FLAG_DROP_RECOVER)
#endif


// --- VoiceChat mode.
/* @enum VCTMode
*  @brief Enumerates chat modes. */ 
typedef enum {
    VCT_MODE_NULL,
    VCT_MODE_CONFERENCE //!< Conference mode.
} VCTMode;

/*!
@brief Enumerates session states.
 */
typedef enum {
    VCT_STATE_INIT,    //!< Specifies an initialized state.
    VCT_STATE_TALKING  //!< Specifies that the peers are talking.
} VCTState;

/*!
@brief Enumerates voice chat library events.
 */
typedef enum {
    VCT_EVENT_NONE,
    VCT_EVENT_CONNECTED,      //!< Connected to the remote client.
    VCT_EVENT_DISCONNECTED    //!< Disconnected.
} VCTEvent;

/*!
@brief Enumerates audio codecs.
 */
typedef enum {
    VCT_CODEC_8BIT_RAW,   //!< Specifies 8-bit 8 kHz raw audio.
    VCT_CODEC_G711_ULAW,  //!< Specifies G711 u-Law.
    VCT_CODEC_2BIT_ADPCM, //!< Specifies 2-bit IMA-ADPCM (non-standard).
    VCT_CODEC_3BIT_ADPCM, //!< Specifies 3-bit IMA-ADPCM.
    VCT_CODEC_4BIT_ADPCM, //!< Specifies 4-bit IMA-ADPCM.

    VCT_CODEC_END
} VCTCodec;

/*!
@brief Enumerates echo cancellation modes.
 */
typedef enum {
    VCT_EC_STANDARD,        //!< Specifies standard echo cancellation.
    VCT_EC_HIGHPERFORMANCE  //!< Specifies high-performance echo cancellation (only available in debug mode).
} VCTECMode;

typedef struct VCTSession_ VCTSession;

// --- Callback function prototypes.

/*!
@brief Defines the type for an event callback.

@see     VCTConfig, VCTSendCallback
 */
typedef void (*VCTEventCallback)( u16 cid, VCTEvent result, VCTSession *session, void *param );

/*!
@brief Defines the type for a send callback.

@see     VCTConfig, VCTEventCallback
 */
typedef bool (*VCTSendCallback) ( u16 cid, const void *buff, u32 size, bool isReliable );

/*!
@brief Stores information related to audio.
*/
typedef struct
{
    VCTCodec     codec;                  //!< Specifies the audio codec of the peer.
    u8           padding0_[3];           //!< Represents a reserved region.

    u32          sequence;               //!< Sequence number of the current audio packet.

#ifdef NN_VCT_TARGET_CAFE
    OSTime       bufferLatency;          //!< Latency of the jitter buffer.
    OSTime       jitter;                 //!< Maximum jitter (in ticks).
    OSTime       clockSkew;              //!< Clock skew (in ticks).
#elif NN_VCT_TARGET_CTR
    nn::os::Tick bufferLatency;          //!< Latency of the jitter buffer.
    nn::os::Tick jitter;                 //!< Maximum jitter (in ticks).
    nn::os::Tick clockSkew;              //!< Clock skew (in ticks).
#endif

    u32          internalLatency;        //!< Internal library latency.
    u32          bufferCount;            //!< Specifies the number of packets in the jitter buffer.
    u32          dropCount;              //!< Specifies the number of packets that were dropped.
    u32          continuousDrop;         //!< Number of consecutive dropped packets in the most recent frame.

    u32          jamCount;               //!< Number of jammed packets.
    u32          duplicateCount;         //!< Specifies the number of duplicate packets.
    u32          waitCount;              //!< Number of packets on hold due to the loss of the previous packet.
    u32          lateCount;              //!< Number of packets that were late due to network traffic or another reason.

    u32          recentLateCount;        //!< Number of packets that arrived late in the most recent frame.
    u32          recentReceiveCount;     //!< Number of packets that arrived in the most recent frame.
    u32          recentLateRatio;        //!< Ratio of packets that were late (<tt>recentLateCount</tt> / <tt>recentReceiveCount</tt>). 
    u32          recoverCount;           //!< Number of recovered packets.

    u32          recoverUsingSpareCount; //!< Number of packets that were recovered using spare packets.
    u32          padding1_;              //!< Represents a reserved region.
} VCTAudioInfo;

// --- SSP session information.
/*!
@brief Defines the structure for a session. Used for managing CIDs and the session state.

@see VCTConfig
*/
typedef struct VCTSession_
{
    VCTMode     mode;                     //!< Chat mode in this session.
    u8          _padding0_;               //!< Represents a reserved region.
    u16         cid;                      //!< Remote CID.
    u16         _padding1_;               //!< Represents a reserved region.
    u16         cidList[VCT_MAX_CLIENT];  //!< List of client CIDs in conference mode.
    VCTState    state;                    //!< State of this session.
    u8          _padding2_;               //!< Represents a reserved region.
    void*       userData;                 //!< User data.

    VCTSession* _next;
} VCTSession;

/*!
@cond PRIVATE  
@brief Stores data related to the biquad filter.
*/
typedef struct vct_biquad_data
{
  s16 input1;
  s16 input2;
  s16 output1;
  s16 output2;
} vct_biquad_data;
//! @endcond


/*!
@cond PRIVATE
@brief Stores information related to the VAD (Voice Activity Detection) feature.
*/
typedef struct VCTVADInfo
{
    u32         scale;          //!< Scale for input audio (0-127).
    u32         releaseCount;   //!< VAD release count.
    u32         releaseTime;    //!< VAD release time.
    bool        activity;       //!< <tt>true</tt> if VAD detects sound, or <tt>false</tt> if VAD detects noise.
    u8          _padding_[3];   //!< Reserved.
} VCTVADInfo;
//! @endcond

/*!
@brief Stores initialization parameters for the VCT library.

@see VCT_Init

@date 2011/10/21  Added the <tt>ecMode</tt> member to specify the operation mode of echo cancellation.
*/
typedef struct
{
    VCTSession*              session;           //!< Specifies the region to save the <tt>VCTSession</tt> structure in.
    u32                      numSession;        //!< Specifies the size of the region specified in <tt>session</tt>.
    VCTMode                  mode;              //!< Specifies the chat mode (always specify <tt>VCT_MODE_CONFERENCE</tt>).
    VCTECMode                ecMode;            //!< Specifies the operation mode of echo cancellation.
    u16                      cid;               //!< Specifies the CID of the client.
    u8                       codec;             //!< Specifies the codec for the main packets (always specify <tt>VCT_CODEC_4BIT_ADPCM</tt>).
    u8                       frameLength;       //!< Specifies the audio frame length in milliseconds.
    u8                       flag;              //!< Enables or disables VAD, spare packets, drop recovery, and high-pass filter features.
    u8                       _padding1_;        //!< Represents a reserved region.
    void*                    audioBuffer;       //!< Specifies the receive buffer for the audio stream (requires 32-byte alignment).
    u32                      audioBufferSize;   //!< Specifies the size of the buffer specified in <tt>audioBuffer</tt>.
    VCTEventCallback         callback;          //!< Specifies the event callback function.
    VCTSendCallback          sendCallback;      //!< Specifies the send callback function.
    void*                    userData;          //!< Specifies the user data to pass when calling the callback function.

} VCTConfig;


/*===========================================================================*
 *
* Initialize / Finalize Functions
 *
 *===========================================================================*/
/*!
@name Initialization and Finalization
    @{
*/
    /*!
@brief  This function initializes the VCT library.
      
@param[in] config <tt>@ref VCTConfig</tt> structure.
      
@retval true  Initialization succeeded.
@retval false Initialization failed.
      
@date 2011/08/31  Initial version.
     */
bool VCT_Init ( VCTConfig* config );

    /*!
@brief  Checks whether the VCT library is finished being initialized.

@retval true  Already initialized.
@retval false Pre-initialized state.
      
@date 2011/08/31  Initial version.
     */
bool VCT_IsInitialized  ( void );

    /*!
@brief Finalizes the VCT library.
      
@details  Call this function when finalizing VoiceChat or changing the operation mode (<tt>@ref VCTMode</tt>).

@date 2011/08/31  Initial version.
     */

void VCT_Cleanup        ( void );

    /*!
@brief  This function runs the main loop of the VCT library.
      
@details Sends the audio data queued by <tt>@ref  VCT_SendAudioEx</tt> and decodes the received audio data passed by <tt>@ref  VCT_HandleData</tt>.

@details The process for sending data that takes place inside <tt>VCT_Main</tt> (the calling of the send callback) must be called a sufficient number of times, so you also need to call <tt>VCT_Main</tt> a sufficient number of times.
If the audio frame length is <i>length</i> ms and there are <i>num</i> number of communication partners, you must call it more than (<i>num</i> * 1000 / <i>length</i>) times each second.
For example, if the audio frame length is 144 ms and there are 8 connected systems, the calculation is 7 * 1000 / 144 = 48.6, so you must call it about 50 to 60 times each second.
Also, make calls at regular intervals rather than all at once whenever possible, because audio processing is performed at regular intervals. For example, call it a sufficient number of times in each game frame.
                
@date 2013/06/07 Added a description of the number of times to call <tt>VCT_Main</tt>.
@date 2011/08/31  Initial version.
     
*/
void VCT_Main           ( void );

/*!
end of name Initialization and Finalization
    @}
*/

/*!
@name Per-frame Execution
    @{
*/

    /*!
@brief Processes voice chat data.

@details Pass the data received by the communications library to this function.

@note If the return value is <tt>true</tt> (that is, the received data is voice chat data), no additional processing is necessary because the callback function specified in the <tt>callback</tt> member variable of the <tt>@ref  VCTConfig</tt> structure is called from this function.  
If the return value is <tt>false</tt>, the received data is something other than voice chat data (game data, for example), and you must implement some means for the application to perform the necessary processing on the data.

@param[in] cid    <tt>ConnectionID</tt> of the data sender.
@param[in] buffer Pointer to the start of the received data.
@param[in] size   The size of the received data. This value is in bytes.

@retval true  The passed data is voice chat data.
@retval false Either the passed data is voice chat data, or the VCT library is not initialized.

@date 2013/06/07 Revised the description.
@date 2011/08/31  Initial version.
     


*/
bool VCT_HandleData     ( u16 cid, u8* buffer, u32 size );

/*!
end of name Per-frame Execution
    @}
*/

    /*!
@brief  Deletes the specified session.

@param[in] session The <tt>@ref VCTSession</tt> structure to delete.
@retval true  Deletion succeeded.
@retval false Deletion failed.

@date 2011/08/31  Initial version.
     */
bool        VCT_DeleteSession          ( VCTSession *session );

    /*!
@brief  Adds a client to the conference.

@details One's own connection ID does not need to be specified. (It is ignored internally by the function even if it is specified.) To start a conference, this function must be called by all parties for each other. For example, for three systems with the connection IDs 1, 2, and 3 to start a conference, system 1 must add 2 and 3, system 2 must add 1 and 3, and system 3 must add 1 and 2.

@details If N is the maximum number of people who can connect in a conference, the maximum number of clients that the local system can add to a conference is N - 1. Before adding clients you must use the <tt>@ref VCT_Init</tt> function to specify x number of <tt>@ref VCTSession</tt> structures, where x = (total number of participants - 1). If there are not enough sessions, the function returns an error.

@details An error also occurs if this function is called and the mode is not <tt>VCT_MODE_CONFERENCE</tt>.

@param[in] cid The connection ID of the client being added. 


@retval 0  Indicates that the connection ID of the client was added successfully.
@retval Negative  The error code when adding the connection ID of the client failed. 

@date 2013/06/11 Fixed a mistake in the description of the return value.
@date 2011/08/31  Initial version.
     */
s32         VCT_AddConferenceClient    ( u16 cid );

    /*!
@brief  Removes a client from the conference.

@details Call this function for all participants in the conference, and not just the person being removed.

@param[in] cid The connection ID of the client to remove. 


@retval 0  Indicates that the connection ID of the client was removed successfully.
@retval Negative  The error code that is returned when removal of the connection ID of the client fails.
@date 2011/08/31  Initial version.
     */

s32         VCT_RemoveConferenceClient ( u16 cid );
    /*!
@brief  Returns whether a client is in the conference.

@param[in] cid The connection ID of the client to check for membership in the conference. 

@retval true  Participating.
@retval false  Not a member.

@date 2011/08/31  Initial version.
     */
bool        VCT_OnConferenceClient     ( u16 cid );

/*===========================================================================*
 *
* Audio Streaming Functions
 *
 *===========================================================================*/
/*!
@name Audio Streaming
    @{
*/
    /*!
@brief  Starts audio streaming.

@details Call this function after the session has been established.

@param[in] session The session for which to start streaming.

@retval true    Success.
@retval false   Failed.
      
@date 2011/09/29 - Clarified that the <tt>session</tt> variable is <tt>const</tt>.
@date 2011/08/31  Initial version.
     */


bool     VCT_StartStreaming      ( const VCTSession *session );


    /*!
@brief  Clears audio streaming.

@param[in] session The session for which to clear streaming. 

@date 2013/02/15  Initial version.
     */

void VCT_ClearStreaming( const VCTSession *session );
    /*!
@brief  Stops audio streaming.

@param[in] session The session for which to stop streaming. 

@date 2011/09/29  The <tt>session</tt> variable was declared <tt>const</tt>.
@date 2011/08/31  Initial version.
     */
void     VCT_StopStreaming       ( const VCTSession *session );

#ifdef NN_VCT_TARGET_CTR
    /*!
@brief  Queues audio data in the send queue.

@details  This function queues audio data in the send queue. This function also handles echo canceler processing.

@details VAD processing, audio encoding, and the sending of data packets are all performed in the <tt>@ref VCT_Main</tt> function.

@details The data in <span class="argument">audio_data</span> must have been sampled at 16 bits/8 kHz and aligned to a 32-byte boundary. Calling this function once queues data for a single audio frame using this pointer as the starting point To get the corresponding size of a single audio frame, use the <tt>@ref VCT_GetAudioDataSize</tt> function.

@details For <span class="argument">micScale</span>, specify the volume scale value that was returned by the <tt>@ref VCT_GetAudioScale</tt> function.

@details For <span class="argument">micOffset</span>, specify the number of samples that is the computed difference between the newest microphone sample at the time of function execution and the last sample of microphone data input to this function. Specify <tt>0</tt> to input the most recent microphone sample to this function.

@details This function can be called regardless of whether audio streaming has started. If audio streaming has not yet started, this function does nothing.

@details If this function is called at varying intervals, the timestamps on the send packets also vary, and the receiving system experiences more errors. If at all possible, call this function in precise multiples of the currently set audio frame length. (Use a timer or other means to call this function in units of audio frames.) Use the <tt>@ref VCT_GetAudioFrameLength</tt> function to get the currently set audio frame length.

@details <b>Note:</b> Use the <tt>@ref VCT_GetVADActivity</tt> function to get the VAD determination result.


@param[in] audio_data The audio data.
@param[in] micScale The volume scale value for the audio data.
      
@param[in] micOffset  Specifies the offset (number of samples) between the latest microphone sample and last sample of the input microphone data.

@retval true  The audio data has been queued in the send queue. 
@retval false  The audio data has not been queued in the send queue. (Streaming has not started, or the library is not initialized.) 

@date 2011/09/29 The <tt>session</tt> variable was declared <tt>const</tt>.
@date 2011/08/31  Initial version.
     */
#elif NN_VCT_TARGET_CAFE
    /*!
@brief  Queues audio data in the send queue.

@details  This function queues audio data in the send queue. It also performs echo cancellation.

@details VAD processing, audio encoding, and the sending of data packets are all performed in the <tt>@ref VCT_Main</tt> function.

@details The data in <span class="argument">audio_data</span> must have been sampled at 16 bits/8 kHz and aligned to a 32-byte boundary. Calling this function once queues data for a single audio frame using this pointer as the starting point. To get the corresponding size of a single audio frame, use the <tt>@ref VCT_GetAudioDataSize</tt> function.

@details For <span class="argument">micScale</span>, specify the volume scale value that was returned by the <tt>@ref VCT_GetAudioScale</tt> function.

@details For <span class="argument">micOffset</span>, specify the number of samples that is the computed difference between the newest microphone sample at the time of function execution and the last sample of microphone data input to this function. Specify <tt>0</tt> to input the most recent microphone sample to this function.

@details This function can be called regardless of whether audio streaming has started. If audio streaming has not started, this function does nothing.

@details If this function is called at varying intervals, the timestamps on the send packets also vary, and the receiving system experiences more errors. If at all possible, call this function in precise multiples of the currently set audio frame length. (Use a timer or other means to call this function in units of audio frames.) Use the <tt>@ref VCT_GetAudioFrameLength</tt> function to get the currently set audio frame length.

@details <b>Note:</b> Use the <tt>@ref VCT_GetVADActivity</tt> function to get the VAD determination result.


@param[in] audio_data The audio data.
@param[in] micScale The volume scale value for the audio data.
      
@retval true  The audio data has been queued in the send queue. 
@retval false  The audio data has not been queued in the send queue. (Streaming has not started, or the library is not initialized.) 

@date 2011/09/29 The <tt>session</tt> variable was declared <tt>const</tt>.
@date 2011/08/31  Initial version.
     */

#endif

#ifdef NN_VCT_TARGET_CTR
bool     VCT_SendAudioEx         ( const void *audio_data, u32 micScale, u32 micOffset );
#endif
#ifdef NN_VCT_TARGET_CAFE
bool     VCT_SendAudioEx         ( const void *audio_data, u32 micScale);
#endif
    /*!
@brief Extracts audio data from the receive buffer.

@details  The <tt>@ref VCT_HandleData</tt> function decodes the received audio data.
In Conference Mode, this function also acts internally to mix the audio if it is playing on more than one channel.

@details  The audio data written to <span class="argument">audio_data</span> is in 16 bit/8 kHz format. This buffer must be aligned to a 32-byte boundary.

@details This function can be called regardless of whether audio streaming has started or not.
If audio streaming has not started, the function clears the <span class="argument">audio_data</span> buffer to all zeroes.

@details If this function is called at varying intervals, breaks in the audio are more likely. If at all possible, call this function in precise multiples of the currently set audio frame length. (Use a timer or other means to call it in units of audio frames.)
Use the <tt>@ref VCT_GetAudioFrameLength</tt> function to get the currently set audio frame length.

@details This currently set audio frame length is the basis of the data size that is calculated for the length of audio data to write to the buffer.
For <span class="argument">length</span>, specify the same value as this data size. If you specify any other value, data is not fetched.
Use the <tt>@ref VCT_GetAudioDataSize</tt> function to get the data size calculated based on the audio frame length that is currently set.

@details Specify <span class="argument">outCIDList</span> to get a list of the remote connection IDs that sent the stream.
Use this option in Transceiver mode to do things such as switching the pan according to the person who is talking.

@details  You can use the value returned by this function to determine whether there is any audio currently to play. 
This can be used to mute or turn down the volume of BGM when audio is playing.

@param[out] audio_data Pointer to the audio data buffer. 
@param[in] length Size of the audio data buffer.
@param[out] outCIDList A list of connection IDs for clients that have sent this stream. It is possible to specify <tt>NULL</tt>. 


@retval true  Audio data was written to the buffer.
@retval false  Data is not audio data, streaming has not started, or the library is not initialized. 

@date 2012/04/03 - Revised the description of the <SPAN class="argument">outCIDList</SPAN> argument.
@date 2011/08/31  Initial version.
     
*/
bool     VCT_ReceiveAudio        ( void *audio_data, u32 length, u16 outCIDList[] );

    /*!
@brief Sets the audio codec.

@details The codec can be changed even while streaming.

@details The only supported codecs are 4-bit ADPCM and 2-bit ADPCM. Returns <tt>false</tt> when an unsupported audio codec is specified.

@details Use the <tt>@ref VCT_SetSpareCodec</tt> function to change the audio codec for spare packets.

@param[in] codec The changed audio codec type for the sender. 


@retval true  Audio data was written to the buffer.
@retval false  Data is not audio data, streaming has not started, or the library is not initialized.
@date 2011/08/31  Initial version.
     */
bool     VCT_SetCodec            ( VCTCodec codec );

    /*!
@brief Gets the currently configured audio codec.

@details Use the <tt>@ref VCT_GetSpareCodec</tt> function to get the audio codec type for spare packets.



@retval @ref VCTCodec  The audio codec that is currently set for the main packet. 
@date 2011/08/31  Initial version.
     */
VCTCodec VCT_GetCodec            ( void );

    /*!
@brief  Determines whether the main packet audio codec type specified by <span class="argument">codec</span> is usable.

@param[in] codec Audio codec type. 

@retval true  Specifies that the codec can be used.
@retval false  Specifies that the codec cannot be used.

@date 2011/08/31  Initial version.
     */
bool     VCT_IsValidCodec        ( VCTCodec codec );

    /*!
@brief Sets the audio frame length.

@details For <span class="argument">frameLength</span>, you must specify a value between <tt>@ref  VCT_AUDIO_FRAME_LENGTH_MIN </tt> and <tt>@ref  VCT_AUDIO_FRAME_LENGTH_MAX</tt> that is also an integer multiple of 8.

@param[in] frameLength  Audio frame length (in milliseconds). 


@retval true  Configuration succeeded.
@retval false  Configuration failed.
      
@date 2011/08/31  Initial version.
     */
bool     VCT_SetAudioFrameLength ( u32 frameLength );

    /*!
@brief Gets the currently configured audio frame length.

@details Returns the value set in the <span class="argument">frameLength</span> member of the <tt>@ref VCTConfig</tt> structure at time of initialization, or the value set by the <tt>@ref VCT_SetAudioFrameLength</tt> function after initialization.

@details If you call this function before initialization, the recommended value <tt>@ref  VCT_AUDIO_FRAME_LENGTH_DEFAULT</tt> is returned.

@retval true  Configuration succeeded. 
@retval false  Configuration failed. 

@date 2011/08/31  Initial version.
     */
u32      VCT_GetAudioFrameLength ( void );

    /*!
@brief Gets the packet size of audio data.

@details This function returns the data size calculated based on the currently configured audio frame length. You can use the <tt>@ref VCT_GetAudioFrameLength</tt> function to get the currently set audio frame length.

@details If you call this function before initialization, the data size <tt>@ref  VCT_AUDIO_DATA_SIZE_DEFAULT</tt> corresponding to the recommended audio frame length <tt>@ref  VCT_AUDIO_FRAME_LENGTH_DEFAULT</tt> is returned.


@retval u32  Packet size of audio data. 
@date 2011/08/31  Initial version.
     */
u32      VCT_GetAudioDataSize    ( void );

    /*!
@brief Gets the calculated audio scale (square of the average power) of the input audio data.

@details Before using the <tt>@ref VCT_SendAudioEx</tt> function, use this function to get the volume scale value of the input audio data.

@details The <SPAN class="argument">length</SPAN> argument specifies the number of bytes of audio data to input. The audio data must have been sampled at 16 bits/8 kHz.

@param[in] audio  Specifies the audio data. 
@param[in] length  Specifies the number of bytes of the audio data.  

@retval true  The received data is VoiceChat data.

@date 2012/04/19 - Corrected the description of the <SPAN class="argument">length</SPAN> argument by changing "number of samples" to "number of bytes."
@date 2011/09/29 - Clarified that the <span class="argument">audio variable</span> is <tt>const</tt>.
@date 2011/08/31  Initial version.
     */
u32      VCT_GetAudioScale       ( const void *audio, u32 length );
    /*!
@brief  Gets audio streaming information for the party specified by <SPAN class="argument">cid</SPAN>.

@param[in] cid The connection ID of the other party from which to get audio streaming information.
@param[in] info Pointer to an <tt>@ref VCTAudioInfo</tt> structure. 

@date 2011/08/31  Initial version.
     */
void     VCT_GetAudioInfoFromCid ( u16 cid, VCTAudioInfo *info );

    /*!
@cond PRIVATE
@brief  Resets audio streaming information for the terminal that has the specified connection ID.

@details Use the <tt>@ref VCT_ResetAudioInfoAll</tt> function to reset all currently open streams.

@param[in] cid The connection ID of the party for which to reset streaming information. 

@date 2011/08/31  Initial version.
     */
void     VCT_ResetAudioInfo      ( u16 cid );
//! @endcond

    /*!
@cond PRIVATE
@brief  Resets the information on all currently open audio streams.

@details Use the <tt>@ref VCT_ResetAudioInfo</tt> function to reset the streaming information for a specific connection ID.

@date 2011/08/31  Initial version.
     */
void     VCT_ResetAudioInfoAll   ( void );
//! @endcond

/*!
end of name Audio Streaming
    @}
*/

/*!
@name Noise Skipping
    @{
*/

/*!
@brief  Sets the length of time for which sound is not sent in <tt>@ref VCT_SignalNoiseSkip</tt>.

@param[in] count Specifies the length of time sound is not sent (in milliseconds). 
@date 2012/03/01  Initial version.
     */
void     VCT_SetNoiseSkipTime    ( u32 count );

    /*!
@brief  Returns the length of time for which sound is not sent in <tt>@ref VCT_SignalNoiseSkip</tt>.

@retval u32  The length of time for which sound is not sent (in milliseconds).

@date 2012/03/01  Initial version.
     */
u32      VCT_GetNoiseSkipTime    ( void );

    /*!
@brief  When VAD is enabled and not active, this function makes VAD active on the sound of button presses so that audio is not sent.
      
@details Setting up this function enables you to prevent the VAD feature from activating and sending audio on the sound of button presses when VAD is enabled but not active.
By not sending audio for a certain time period after button presses, you can minimize the button pressing sounds received by other players.

@details This function cannot remove button-press sounds while VAD is active and a conversation is being sent.

@details The duration (in milliseconds) to not send sound is specified with <tt>@ref VCT_SetNoiseSkipTime</tt>.


@date 2012/05/10 - Revised the description.
@date 2012/03/15 - Supplemented the description.
@date 2012/03/01  Initial version.
     */

void     VCT_SignalNoiseSkip     ( void );

/*!
end of name Noise Skipping
    @}
*/

/*!
@name Spare Packets
    @{
*/
/*===========================================================================*
 *
* Spare Packet Functions
 *
 *===========================================================================*/

    /*!
@brief  This function enables and disables the spare packet feature.

@details The spare packet feature is enabled by default.

@param[in] flag  Specify <tt>true</tt> to enable or <tt>false</tt> to disable. 

@date 2011/08/31  Initial version.
     */
void     VCT_EnableSparePacket   ( bool flag );

    /*!
@brief  This function gets whether the spare packet feature is enabled or disabled.

@details The spare packet feature is enabled by default.

@retval true  Specifies that the spare packet feature is enabled. 
@retval false  Specifies that the spare packet feature is disabled.
      
@date 2011/08/31  Initial version.
     */
bool     VCT_IsEnableSparePacket ( void );

    /*!
@brief Sets the audio codec for spare packets.

@details By default, spare packets use the 2-bit ADPCM codec.

@details Use the <tt>@ref VCT_SetCodec</tt> function to change the codec for main packets.

@details <b>Note:</b> The only codec type supported for spare packets is 2-bit ADPCM. Accordingly, this function does not need to be called at this time. Returns <tt>false</tt> when an unsupported codec has been specified.

@param[in] spareCodec The spare packet codec type.  

@retval true  Setting succeeded. 
@retval false  Setting failed.

@date 2011/08/31  Initial version.
     */
bool     VCT_SetSpareCodec       ( VCTCodec spareCodec );

    /*!
@brief Gets the currently configured audio codec for spare packets.

@details This value is meaningless if the spare packet feature is not used.

@details <b>Note:</b> The only codec type supported for spare packets is 2-bit ADPCM. At the present time, this function always returns <tt>@ref VCT_CODEC_2BIT_ADPCM</tt>.

@retval @ref VCTCodec  The currently set codec type for spare packets.  

@date 2011/08/31  Initial version.
     */
VCTCodec VCT_GetSpareCodec       ( void );

    /*!
@brief Determines whether you can use the codec type specified by <span class="argument">codec</span> as a spare-packet codec type.

@param[in] codec The codec type.  

@retval true  Can be used.
@retval false  Cannot be used.

@date 2011/08/31  Initial version.
     */
bool     VCT_IsValidSpareCodec   ( VCTCodec codec );

/*!
end of name Spare Packets
    @}
*/
/*!
@name Drop Recovery
    @{
*/
/*===========================================================================*
 *
* Drop Recovery
 *
 *===========================================================================*/

/*!
@cond PRIVATE
@brief Enables or disables the drop recover feature.

@details The drop recover feature is enabled by default.

@details When the drop recovery feature is enabled, an attempt is made to recover audio data from lost packets.

@details If the spare packet feature is enabled and the spare packet of the next packet is available, the audio data of the current packet is recovered based on the spare packet audio data. This improves audio quality.

@param[in] flag  Specify <tt>true</tt> to enable or <tt>false</tt> to disable. 

@date 2011/08/31  Initial version.
     */
void VCT_EnableDropRecover   ( bool flag );
//! @endcond

/*!
@cond PRIVATE
@brief Gets whether the drop recover feature is enabled.

@retval true  Specifies that the drop recover feature is enabled.    
@retval false  Specifies that the drop recover feature is disabled.   

@date 2011/08/31  Initial version.
     */
bool VCT_IsEnableDropRecover ( void );
//! @endcond
/*!
end of name Drop Recovery
    @}
*/
/*!
@name VAD
    @{
*/
/*===========================================================================*
 *
* VAD Functions
 *
 *===========================================================================*/
/*!
@brief Enables or disables the VAD feature.

@details The VAD feature is enabled by default.

@details When VAD is enabled, in addition to evaluating audio inside the <tt>@ref  VCT_Main</tt> function, decisions are also made automatically about whether to send packets.

@param[in] flag  Specify <tt>true</tt> to enable or <tt>false</tt> to disable.  

@date 2011/08/31  Initial version.
     */
void VCT_EnableVAD               ( bool flag );

/*!
@brief Gets whether the VAD feature is enabled.

@retval true  Specifies that the VAD feature is enabled. 
@retval false  Specifies that the VAD feature is disabled.  

@date 2011/08/31  Initial version.
     */
bool VCT_IsEnableVAD             ( void );

/*!
@brief VAD Gets the audio detection results from the VAD algorithm.

@details Returns <tt>true</tt> while audio is detected in the frame (speech frame) and <tt>false</tt> while it is not (silent frame).

@details Always returns <tt>false</tt> while the VAD feature is disabled, regardless of the presence of speech.

@retval true  Specifies that audio is present. 
@retval false  Specifies that no audio is present.  

@date 2011/08/31  Initial version.
     */
bool VCT_GetVADActivity          ( void );

/*!
@cond PRIVATE
@brief Gets the average power of audio data in the current audio frame.

@details The VAD algorithm detects audio based on the size of this value. This value is equal to the average power of the entire audio data sample (the square root of the audio data volume scale value used by the <tt>@ref VCT_GetAudioScale</tt> and <tt>@ref VCT_SetVADClampGain</tt> functions) divided by a normalizing factor (256).

@retval u16 Average power of the audio data in the current audio frame. 

@date 2011/08/31  Initial version.
     */
u16  VCT_GetVADPower             ( void );
//! @endcond

/*!
@brief Resets the internal VAD status.

@details Call this function after making changes to the microphone gain or similar settings.


@date 2011/08/31  Initial version.
     */
void VCT_ResetVAD                ( void );

/*!
@brief Sets the speech frame detection threshold.

@details When the previous audio frame is recognized as a silent frame, this threshold value is used to determine whether the current audio frame contains speech.

@details A frame is recognized as containing speech when the following condition is met, where Ts is the threshold value, Ec is the average speech power of the audio data in the current frame, and En is the average speech power of the audio data in the four previous frames.

@details Ec > En * Ts / 100

@details The value argument takes the threshold (Ts). The default value is 350. In this case, the threshold of the power difference between <tt>En</tt> and <tt>Ec</tt> is equivalent to <tt>20 * log (350 / 100) = 10.9 dB</tt>.

@param[in] value  Specifies the speech frame detection threshold. 

@date 2011/08/31  Initial version.
     */
void VCT_SetVADActiveGain        ( u16 value );

/*!
@brief Gets the current speech frame detection threshold.

@retval u16 Threshold for detecting a speech frame. 

@date 2011/08/31  Initial version.
     */
u16  VCT_GetVADActiveGain        ( void );

/*!
@brief Sets the silent frame detection threshold.

@details When the previous audio frame is recognized as a speech frame, this threshold value is used to determine whether the current audio frame is silent.

@details A frame is recognized as being a silent frame when the following condition is met, where Tn is the threshold value, Ec is the average speech power of the audio data in the most-recent four frames, and En is the average speech power of the audio data in the four frames prior to the last audio frame that was determined to be a silent frame.

@details Ec < En * Tn / 100

@details  However, the sending of audio frames actually stops when silence is determined to continue for the consecutive number of frames specified by the <tt>@ref  VCT_SetVADReleaseTime</tt> function.

@details The value argument takes the threshold (Tn). The default value is 280. In this case, the threshold of the power difference between <tt>En</tt> and <tt>Ec</tt> is equivalent to <tt>20 * log (280 / 100) = 8.94 dB</tt>.

@param[in] value  Specifies the silent frame detection threshold.  

@date 2011/08/31  Initial version.
     */
void VCT_SetVADInactiveGain      ( u16 value );
/*!
@brief Gets the current silent frame detection threshold.

@retval u16 Threshold for detecting a silent frame. 

@date 2011/08/31  Initial version.
     */
u16  VCT_GetVADInactiveGain      ( void );

/*!
@brief Sets the time that must elapse after silence is detected before packet transmission is stopped.

@details To prevent VAD from detecting silence in the middle of a conversation because, for example, the user takes a breath, set the release time to at least 500 ms.

@details For <SPAN class="argument">count</SPAN>, specify the time (in packets) to wait after VAD has detected silence before packet transmission is halted. The default is 5 packets (<tt>144 * 5 = 720 ms</tt>).

@param[in] count Release time (in number of packets). 

@date 2011/08/31  Initial version.
     */
void VCT_SetVADReleaseTime       ( u32 count );

/*!
@brief Gets the time that must elapse after silence is detected before packet transmission is stopped.

@retval u32 Release time (in number of packets). 

@date 2011/08/31  Initial version.
     */
u32  VCT_GetVADReleaseTime       ( void );

/*!
@brief Sets the volume threshold as a volume scale value (the square of the average power) for unconditionally determining silence.

@details <span class="argument">value</span> takes a volume scale value in a range from <tt>0</tt> through <tt>0x3FFFFFFF</tt>. Any audio frame with a volume scale value below this specified threshold is determined to be a silent frame.

@details Uses the volume scale value for audio data obtained by the <tt>@ref VCT_GetAudioScale</tt> function.

@details The default value is 49.

@param[in] value  The volume threshold at which a frame is unconditionally determined to be silent. 
@date 2011/08/31  Initial version.
     */
void VCT_SetVADClampGain         ( u32 value );

/*!
@brief Gets the volume threshold as a volume scale value for unconditionally determining silence.

@details The volume scale value is specified in the range from <tt>0</tt> through <tt>0x3FFFFFFF</tt>.

@retval u32 The volume threshold at which a frame is unconditionally determined to be silent.
  
@date 2011/08/31  Initial version.
     */
u32  VCT_GetVADClampGain         ( void );

/*!
end of name VAD
    @}
*/

/*!
@name HPF
    @{
*/
/*===========================================================================*
 *
* HPF Functions
 *
 *===========================================================================*/
/*!
@cond PRIVATE
@brief Enables or disables the high-pass filter.

@details By default, the high-pass filter is disabled.

@details When this feature is enabled, the high-pass filter is applied inside the <tt>@ref  VCT_Main</tt> function to eliminate low-range noise from the audio being sent. 

@param[in] flag  Specify <tt>true</tt> to enable or <tt>false</tt> to disable.  

@date 2011/08/31  Initial version.
     */
void VCT_EnableHPF   ( bool flag );
//! @endcond

/*!
@cond PRIVATE
@brief Gets whether the high-pass filter is enabled.

@retval true  The high-pass filter feature is enabled.
@retval false  The high-pass filter feature is disabled. 

@date 2011/08/31  Initial version.
     */
bool VCT_IsEnableHPF ( void );
//! @endcond
/*!
end of name HPF
    @}
*/

/*!
@name BPF
    @{
*/
/*===========================================================================*
 *
* BPF Functions
 *
 *===========================================================================*/

/*!
@cond PRIVATE
@brief Enables or disables the biquad filter.

@details By default, the biquad filter is disabled.

@details When this feature is enabled, the biquad filter is applied inside the <tt>@ref  VCT_Main</tt> function to eliminate noise outside of the range of speech from the audio being sent. 


@param[in] flag  Specify <tt>true</tt> to enable or <tt>false</tt> to disable.  

@date 2011/12/09  Initial version.
     */
void VCT_EnableBPF   ( bool flag );
//! @endcond

/*!
@cond PRIVATE
@brief Gets whether the biquad filter is enabled.

@retval true  The biquad filter is enabled.
@retval false  The biquad filter is disabled. 

@date 2011/12/09  Initial version.
     */
bool VCT_IsEnableBPF ( void );
//! @endcond

/*!
end of name HPF
    @}
*/


#ifdef NN_VCT_TARGET_CTR
/*!
@name Echo Canceler
    @{
*/
/*===========================================================================*
 *
* Echo-Canceler Functions
 *
 *===========================================================================*/
/*!
@cond PRIVATE
@brief Enables or disables the echo canceler.

@details The echo canceler feature is enabled by default.

@details When this feature is enabled, echo cancellation is performed inside the <tt>@ref  VCT_SendAudioEx</tt> function.

@param[in] flag  Specify <tt>true</tt> to enable or <tt>false</tt> to disable.  

@date 2013/06/11 Changed the default from disabled to enabled.
@date 2011/08/31  Initial version.
     */
void VCT_EnableEchoCancel   ( bool flag );
//! @endcond

/*!
@cond PRIVATE
@brief Gets whether the echo canceler is enabled.

@retval true  The echo canceler is enabled.
@retval false  The echo canceler is disabled. 
@date 2011/08/31  Initial version.
     */
bool VCT_IsEnableEchoCancel ( void );
//! @endcond

/*!
@brief Supplies reference data for the echo canceler.

@details Call the <tt>nn::snd::GetMixedBusData</tt> function at every sound DSP interrupt (every 5 ms), and pass the 5 ms of 16 bit/32 kHz/stereo data obtained by that function to the <span class="argument">buffer</span> of this function.

@details This function does nothing if <SPAN class="argument">size</SPAN> is not the correct size for 5 milliseconds of 16-bit/32-kHz stereo data.

@param[in] buffer Reference data for the echo canceler.  
@param[in] size The size of the reference data for the echo canceler.   


@date 2011/09/29 - Clarified that the <span class="argument">buffer</span> variable is <tt>const</tt>.
@date 2011/08/31  Initial version.
     */

void VCT_HandleReference    ( const u8 *buffer, u32 size );
/*!
end of name Echo Canceller
    @}
*/
#endif // NN_VCT_TARGET_CTR

#ifdef __cplusplus
}
#endif

#endif /* VCT_VOICECHAT_H_ */
/*!!!! EOF !!!!*/
