/*---------------------------------------------------------------------------*
  Project:  Voice Chat Sample.
  File:     utilities.h

  Copyright (C)2013-2014 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#ifndef SAMPLE_UTILITIES_H_
#define SAMPLE_UTILITIES_H_

#include "definitions.h"


#if VCT_PLATFORM_CTR
  #include <nn/os.h>
#elif NN_VCT_TARGET_CAFE
  #include <cafe/os.h>
#else
  #error "invalid platform";
#endif


/*---------------------------------------------------------------------------*
  Name        : s_Clip
  Description : Clip Value
  Arguments   : value :input value
              : min   :min value
              : max   :max value
  Returns     : clip value
 *---------------------------------------------------------------------------*/
template <typename T>
T s_Clip( T value, s32 min, s32 max )
{
    if ( value < min )
    {
        value = min;
    }
    else if ( max < value )
    {
        value = max;
    }
    return value;
}


#ifdef VCT_PLATFORM_CTR
  #define SAMPLE_PRINT  NN_LOG
  #define SAMPLE_ASSERT NN_ASSERT
  #define SAMPLE_PANIC  NN_PANIC
#elif NN_VCT_TARGET_CAFE
  #define SAMPLE_PRINT  OSReport
  #define SAMPLE_ASSERT ASSERT
  #define SAMPLE_PANIC  OSFatal
#else
  #error "Illegal platform."
#endif



class CriticalSection
{
public:
    CriticalSection();
   ~CriticalSection();

    void Initialize();
    void Finalize();

    void Lock();
    void Unlock();

private:
#if VCT_PLATFORM_CTR
    nn::os::CriticalSection m_CriticalSection;
#elif NN_VCT_TARGET_CAFE
    OSMutex m_Mutex;
#else
  #error "invlaid platform"
#endif
};

#endif  // end of SAMPLE_UTILITIES_H_
