/*---------------------------------------------------------------------------*
  Project:  Voice Chat Sample.
  File:     vct_sample.h

  Copyright (C)2011-2014 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#ifndef SAMPLE_VCT_SAMPLE_H_
#define SAMPLE_VCT_SAMPLE_H_

#include "definitions.h"

/*---------------------------------------------------------------------------*
    Include
 *---------------------------------------------------------------------------*/
#ifdef VCT_PLATFORM_CTR
#include <nn.h>
#include <nn/os.h>
#include <nn/fnd.h>
#include <nn/hid.h>
#else
#include<cafe.h>
#endif

#include <vct.h>
#include "net.h"
#include "sndmic.h"


/*---------------------------------------------------------------------------*
    Define
 *---------------------------------------------------------------------------*/
#define SAMPLE_VCT_DEFAULT_GAME_MODE   (1000)
#define SAMPLE_VCT_DEFAULT_MAX_CONNECT (8)

/*---------------------------------------------------------------------------*
    Macro
 *---------------------------------------------------------------------------*/
#define ALIGN_4( a )  ((((int)a+3)>>2)<<2)


/*---------------------------------------------------------------------------*
    Enum
 *---------------------------------------------------------------------------*/
/*
    <<< Voice Chat Sequence... >>>
*/
enum SAMPLE_VCT_SEQ_STATUS
{
    SAMPLE_VCT_SEQ_INIT = 0,          /* Voice Chat init sequence.    */
    SAMPLE_VCT_SEQ_PROCESS            /* Voice Chat process sequence. */
};

/*
    <<< Network Setting... >>>
*/
/* Network matchmake system mode. */
enum SAMPLE_VCT_NET_MATCH_MODE
{
    SAMPLE_VCT_NET_MATCH_ANYBODY = 0, /* Matchmake is anybody mode.   */
    SAMPLE_VCT_NET_MATCH_FRIEND,      /* Matchmake is friend mode.    */
    SAMPLE_VCT_NET_MATCH_MODE_MAX
};


/*---------------------------------------------------------------------------*
    Typedef
 *---------------------------------------------------------------------------*/
/* NET Parameters... */
typedef struct
{
    SAMPLE_VCT_NET_MATCH_MODE       netMatchMode;
    u32                       gameMode;
    u32                       maxConnect;
} SampleVctNetParam;

/* System... */
typedef struct
{
    SndMicDevice              *sndmicDev;
    NetworkInfra              *netInfra;
} SampleVctSystem;

/* Global... */
typedef struct
{
    SAMPLE_VCT_SEQ_STATUS       sequence;
    SampleVctSystem             system;
    SampleVctNetParam           netParameter;
    NetInformationStructure     netInformation;
} SampleVctGlobal;


#endif /* SAMPLE_VCT_SAMPLE_H_ */
