/*---------------------------------------------------------------------------*
  Project:  Voice Chat Sample.
  File:     wave.h

  Copyright (C) 2009-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#ifndef SAMPLE_WAVE_H_
#define SAMPLE_WAVE_H_

#include "definitions.h"

/*---------------------------------------------------------------------------*/
/* includes */
#ifdef VCT_PLATFORM_CTR
#include <nn/types.h>
#include <nn/fs.h>
#ifndef NN_BUILD_RELEASE  // HostIO cannot be used in the Release build.
#include <nn/hio.h>
#endif // NN_BUILD_RELEASE
#else
#include<cafe.h>
#endif

/*---------------------------------------------------------------------------*/
/* Typedef */
typedef struct
{
    u32 tag;
    u32 size;
    u32 type;
}
WaveRiff;

typedef struct
{
    u32 tag;
    u32 size;
}
WaveChunk;

typedef struct
{
    u16 formatId;
    u16 channel;
    u32 sampleRate;
    u32 transRate;
    u16 blockSize;
    u16 quantumBits;
}
WaveFmt;

typedef struct
{
    u32 size;
    u8* buf;
}
WaveData;

/*---------------------------------------------------------------------------*/
/* Prototypes */

#ifdef VCT_PLATFORM_CTR
void LoadWaveCore(nn::fs::FileReader*, WaveFmt*, WaveData*, bool);
static inline void LoadWave(nn::fs::FileReader* reader, WaveFmt* fmt, WaveData* data)
{
    LoadWaveCore(reader, fmt, data, false);
}
static inline int GetWaveLength(nn::fs::FileReader* reader)
{
    WaveFmt fmt;
    WaveData data;
    LoadWaveCore(reader, &fmt, &data, true);
    return data.size;
}
#endif

#endif // SAMPLE_WAVE_H_

/*---------------------------------------------------------------------------*/
/* End of file */
/*---------------------------------------------------------------------------*/
