﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Shared_Inet_Common.h

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#ifndef SHARED_INET_COMMON_H_
#define SHARED_INET_COMMON_H_

//n1589: Include here that which uses all of the Shared_Inet header
#include "SampleLib/SampleLib.h"

#include <pia/common.h>
#include <pia/transport.h>
#include <pia/session.h>
#include <pia/inet.h>

/* Error for sample */
PIA_DEFINE_RESULT(
    SampleResultInvalidState,
    nn::Result::LEVEL_USAGE,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    1
);

// nn::Result definitions for generally categorizing errors not expressed by nn::Result in the sample.
enum SAMPLE_RESULT_DESCRIPTION {
    FAILED_NETWORK_IFUP = 1,            // 
    FAILED_FRIENDS_LOGIN,               // The login-related processing to the friend server failed.
    FAILED_NEX_LOGIN,                   // The login-related processing to the NEX server failed.
    FAILED_BIND_CREDENTIALS,            // Failed to bind certificate.
    FAILED_AUTO_MATCHMAKE_SESSION,      // 
    FAILED_CREATE_MATCHMAKE_SESSION,    // Failed at process for creating a matchmaking session.
    FAILED_JOIN_MATCHMAKE_SESSION,      // Failed at process for joining a matchmaking session.
    FAILED_OPEN_PARTICIPATION,          // Failed at OpenParticipation.
    FAILED_GET_HOST_CONNECTION_INFO,    // Failed at GetHostStationConnectionInfo.
    SESSION_STATE_DISCONNECTED,         // Disconnected from the session.
    CONNECTION_STATE_DISCONNECTED,      // Network connection lost.
    NEX_SERVER_STATE_DISCONNECTED,      // Connect to NEX server lost.
    NEX_INTERNAL_ERROR                  // NEX error.
};

PIA_DEFINE_RESULT(
    SampleFailedNetworkInterfaceUp,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    FAILED_NETWORK_IFUP
);

PIA_DEFINE_RESULT(
    SampleFailedFriendsLogin,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    FAILED_FRIENDS_LOGIN
);

PIA_DEFINE_RESULT(
    SampleFailedNexLogin,               
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    FAILED_NEX_LOGIN
);

PIA_DEFINE_RESULT(
    SampleFailedBindCredentials,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    FAILED_BIND_CREDENTIALS
);

PIA_DEFINE_RESULT(
    SampleFailedAutoMatchmakeSession,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    FAILED_AUTO_MATCHMAKE_SESSION
);

PIA_DEFINE_RESULT(
    SampleFailedCreateMatchmakeSession,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    FAILED_CREATE_MATCHMAKE_SESSION
);

PIA_DEFINE_RESULT(
    SampleFailedJoinMatchmakeSession,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    FAILED_JOIN_MATCHMAKE_SESSION
);

PIA_DEFINE_RESULT(
    SampleFailedOpenParticipation,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    FAILED_OPEN_PARTICIPATION
);

PIA_DEFINE_RESULT(
    SampleFailedGetHostStationConnectionInfo,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    FAILED_GET_HOST_CONNECTION_INFO
);

PIA_DEFINE_RESULT(
    SampleSessionStateDisconnected,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    SESSION_STATE_DISCONNECTED
);

PIA_DEFINE_RESULT(
    SampleConnectionStateDisconnected,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    CONNECTION_STATE_DISCONNECTED
);

PIA_DEFINE_RESULT(
    SampleNgsStateDisconnected,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    NEX_SERVER_STATE_DISCONNECTED
);

PIA_DEFINE_RESULT(
    SampleNexInternalError,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    NEX_INTERNAL_ERROR
);

#endif
