﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Shared_Nex_Common.h

  Copyright (C) 2012-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#ifndef SHARED_NEX_COMMON_H_
#define SHARED_NEX_COMMON_H_

// References the NN_PIA_NEX_3 value.
#include <pia/settings.h>
#include <pia/platform.h>
#include <nex.h>

// Includes the header corresponding to the NEX major version.
#if NN_PIA_NEX_3

#include <OnlineCore/src/version/nex_version.h>
#include <OnlineCore/src/Core/CallContext.h>

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/GlobalVariables.h>
#include <OnlineCore/src/Platform/Core/SystemErrorResult.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/Type.h>
#include <OnlineCore/src/Platform/Core/CriticalSectionProfiler.h>
#include <OnlineCore/src/Platform/Core/SystemClock.h>
#include <OnlineCore/src/Platform/Core/UpdatableTimeProvider.h>

#include <OnlineCore/src/Platform/Core/ConsoleIO.h>

#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>

#include <OnlineCore/src/Transport/UDP/Packet.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPEndPoint.h>
#include <OnlineCore/src/Transport/Interface/RootTransport.h>
#include <OnlineCore/src/Transport/Interface/SocketTransport.h>
#include <OnlineCore/src/Transport/Interface/TransportError.h>
#include <OnlineCore/src/Transport/Interface/TransportAdapter.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>

#include <NetZ/src/Core/DOCore/Station.h>
#include <NetZ/src/Core/DOCore/Session.h>
#include <NetZ/src/Core/DOCore/SessionDescription.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/DirectStream.h>
#include <NetZ/src/Core/DOCore/DirectTransport.h>
#include <NetZ/src/Extensions/SessionSelection/ConsoleSessionEnumerator.h>
#include <NetZ/HostMigration.h>

#include <RendezVous/Services/NATTraversal/src/Client/NATTraversalClient.h>

#else // not NN_PIA_NEX_3

#include <version/nex_version.h>
#include <Core/CallContext.h>

#include <Platform/Core/MainWrapper.h>
#include <Platform/Core/PlatformDecl.h>
#include <Platform/Core/GlobalVariables.h>
#include <Platform/Core/SystemErrorResult.h>
#include <Platform/Core/qList.h>
#include <Platform/Core/Type.h>
#include <Platform/Core/CriticalSectionProfiler.h>
#include <Platform/Core/SystemClock.h>
#include <Platform/Core/UpdatableTimeProvider.h>

#include <Platform/Core/MultiUnitHeapTemplate.h>
#include <Platform/Core/ErrorUtil.h>

#include <Platform/Stack/Core/InetAddress.h>

#include <Utilities/ConsoleIO/ConsoleIO.h>
#include <Utilities/SessionEnumeration/ConsoleSessionEnumerator.h>

#include <Plugins/Transport/UDP/Packet.h>
#include <Plugins/Transport/PRUDP/PRUDPEndPoint.h>
#include <Plugins/Transport/Interface/RootTransport.h>
#include <Plugins/Transport/Interface/SocketTransport.h>
#include <Plugins/Transport/Interface/TransportError.h>
#include <Plugins/Transport/Interface/TransportAdapter.h>
#include <Plugins/Transport/Interface/StationURL.h>

#include <Plugins/SessionDiscovery/Interface/SessionDescription.h>

#include <ObjDup/Station.h>
#include <ObjDup/Session.h>
#include <ObjDup/DOHandle.h>
#include <ObjDup/DirectTransport.h>
#include <RVPackages/HostMigration.h>

#endif // NN_PIA_NEX_3


// Redefine the definitions that change with the different NEX major version.
#if NN_PIA_NEX_3
#define SAMPLE_SESSION_ROLE_MASTER       nn::nex::SESSION_ROLE_MASTER
#define SAMPLE_SESSION_ROLE_DUPLICA      nn::nex::SESSION_ROLE_DUPLICA
#define SAMPLE_SESSION_ROLE_UNDEFINED    nn::nex::SESSION_ROLE_UNDEFINED
#else
#define SAMPLE_SESSION_ROLE_MASTER       SESSION_ROLE_MASTER
#define SAMPLE_SESSION_ROLE_DUPLICA      SESSION_ROLE_DUPLICA
#define SAMPLE_SESSION_ROLE_UNDEFINED    SESSION_ROLE_UNDEFINED
#endif

// NEX version macro.
#define NEX_VERSION_NUMBER(major, minor, micro) (((major) << 16) | ((minor) << 8)  | ((micro) << 0))
#define NEX_CURRENT_VERSION_NUMBER NEX_VERSION_NUMBER(NEX_VERSION_MAJOR, NEX_VERSION_MINOR, NEX_VERSION_MICRO)

#endif
