﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Shared_Nex_Setup.h

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
#ifndef SHARED_NEX_SETUP_H_
#define SHARED_NEX_SETUP_H_

#include <pia/platform.h>
#include "Shared_Nex_Common.h"

//-----------------------------------------------------------------------------------------
// Declarations of External Functions
//-----------------------------------------------------------------------------------------

nn::Result Nex_InitializeNex(
    size_t memorySize, 
    size_t emergencyMemorySize, 
    nn::nex::Core::ThreadMode threadMode,
    bool useNetZ,
    bool useMultiUnitHeap = false
);
void Nex_FinalizeNex();

// Displays the protocol call context error
void Nex_ReportProtocolCallContext(nn::nex::ProtocolCallContext& context, const char* pHeaderText = NULL);

#if (NN_PIA_NEX_3==0)
// This function is for getting the error reported through the error-handling callback.
nn::nex::qResult& Nex_GetCommonErrorResult();
nn::Result Nex_GetCtrErrorResult();

// Clears the variable for temporarily storing errors reported through the error-handling callback.
void Nex_ClearErrorResult();

// This function is for checking the severity of NEX errors.
bool Nex_CheckErrorSeverity(const nn::nex::qResult& result);
bool Nex_CheckErrorSeverity(nn::Result result);
bool Nex_CheckErrorSeverity(u32 result);
#endif // end of NN_PIA_NEX_3

//-----------------------------------------------------------------------------------------
// Function to get the amount of memory consumed with the multi-unit heap.
//-----------------------------------------------------------------------------------------

// Enable recording peak values
void Nex_EnableMultiUnitHeapPeakRecording (void);

// Disable recording peak values
void Nex_DisableMultiUnitHeapPeakRecording(void);

// Based on the peak value, calculates the minimum size required. Returns 0 when the multi-unit heap is uninitialized.
size_t Nex_GetMultiUnitHeapPeakMemorySize (void);

#endif  // end of _CTRNEXUTIL_H_
