/*---------------------------------------------------------------------------*
  Project:  Voice Chat Sample.
  File:     game_conference.cpp

  Copyright (C) 2011-2014 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*
    Include
 *---------------------------------------------------------------------------*/
#ifdef ENABLE_PROFILING
#include "SampleLib/profiler.h"
#endif

#include "framework.h"
#include "game_conference.h"
#include <vct.h>
#ifdef NN_VCT_TARGET_CAFE
#include <cafe/procui.h>
#endif

#include "utilities.h"


/*---------------------------------------------------------------------------*
    Define
 *---------------------------------------------------------------------------*/
#define MAX_AID (32)
#define CPU_PROFILING_PERIOD 1

/*---------------------------------------------------------------------------*
    Table
 *---------------------------------------------------------------------------*/
static char *enable_str_[] =
{
    "Disable",
    "Enable",
    "",
};
static char *netjoin_str_[] =
{
    "LEAVE   ",
    "PARTICIPATE",
    "",
};
static char *codec_str_[] =
{
    "8bit RAW",
    "G711 u-Law",
    "2bit ADPCM",
    "3bit ADPCM",
    "4bit ADPCM",
    "",
};
static u32 vpos_max_[] =
{
    2,
    14,
#ifdef VCT_PLATFORM_CTR
    9,
#else
    17,
#endif
    16
};


/*---------------------------------------------------------------------------*
    Static
 *---------------------------------------------------------------------------*/
static NetworkInfra            *p_NetInfra;
static SndMicDevice            *p_SndMicDevice;
static u32                     s_MyCID;
static u16                     s_InfoCIDNumber;
static VCTConfig               s_vctConfig;
static VCTSession              s_session[VCT_MAX_SESSION];
#ifdef VCT_PLATFORM_CTR
static u8                      s_audioBuffer[VCT_DEFAULT_AUDIO_BUFFER_SIZE] NN_ATTRIBUTE_ALIGN(32);
#else
static u8                      s_audioBuffer[VCT_DEFAULT_AUDIO_BUFFER_SIZE] __attribute__ ((aligned(32)));
#endif
static u8                      s_joinConferenceList[VCT_MAX_CONFERENCE_CLIENT];
static u32                     s_joinCIDList[VCT_MAX_CONFERENCE_CLIENT];
static u32                     s_joinCIDNum;
static u8                      s_addConferencePage = 0;
static u8                      s_VADValue;
static u8                      s_VADOld;
static bool                    s_BGMFlg;
#ifdef VCT_PLATFORM_CTR
static f32                     s_Volume    = 1.0f;
static f32                     s_BGMVolume = 0.2f;
#else
static u32                     s_Volume    = (SPEAKER_GAIN_IDX_MAX - 1);
#endif
static u32                     s_OldTrig;
static u32                     s_NowTrig;
#ifdef VCT_PLATFORM_CTR
#else
static u32                     s_micGainMax;
static u32                     s_micGainMin;
static u32                     s_micGainStep;
#endif

/*---------------------------------------------------------------------------*
    External
 *---------------------------------------------------------------------------*/
extern u32 ConvertID_Pia2VCT(u32);      //net.cpp
extern u32 ConvertID_VCT2Pia(u32);      //net.cpp
#ifdef VCT_PLATFORM_CTR
#else
extern u32 gHomeMenuStatus;             //vct_sample.cpp
#endif

/*---------------------------------------------------------------------------*
    Prototype
 *---------------------------------------------------------------------------*/
static void dispConference( u32 number, SampleVctGlobal *pGlobal, u32 vpos );
static void dispConference0( u32 vpos );
static void dispConference1( u32 vpos );
static void dispConference2( SampleVctGlobal *pGlobal, u32 vpos );
static void dispConference3( u32 vpos );
static bool configSetting( u32 vpos );
static bool configSetting2( u32 vpos );
static bool addRemoveConferenceMember( u32 vpos, NetInformationStructure *netInfo );
static void startConference( SampleVctGlobal *pGlobal );
static void stopConference( SampleVctGlobal *pGlobal );
static void dispAudioInformation( SampleVctGlobal *pGlobal );
static void EventCallback( u16 connectionID, VCTEvent result, VCTSession *session, void *param );
static bool SendDataCallBack( u16 cid, const void *buff, u32 size, bool isReliable );

/*---------------------------------------------------------------------------*
    Function
 *---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*
  Name        : GameConference
  Description : Main game conference.
  Arguments   : *pGlobal - Render system object.
  Returns     : -
 *---------------------------------------------------------------------------*/
void GameConference( SampleVctGlobal *pGlobal )
{
    nn::pia::framework::Framework* framework = nn::pia::framework::Framework::GetInstance();
    NetInformationStructure   *netInfo       = &pGlobal->netInformation;
    NetInformationStructure   new_netInfo;
    NetworkInfra              *net           = pGlobal->system.netInfra;
    u32                       vpos           = 0;
    u32                       diplay_number  = 0;
    u32                       audio_info_cnt = 0;
    u32                       drawflag;

#ifdef ENABLE_PROFILING
    nn::os::InitializeAlarmSystem();
    // Initialize Profile...
    Profiler::CreateInstance();
#endif

    // Initialize pGlobal...
    p_NetInfra     = pGlobal->system.netInfra;
    p_SndMicDevice = NULL;
    s_MyCID        = netInfo->myCid;
    s_VADValue     = 255;
    s_NowTrig      = 0;

    // Initialize VCT
    s_vctConfig.session         = s_session;
    s_vctConfig.numSession      = VCT_MAX_SESSION;
    s_vctConfig.mode            = VCT_MODE_CONFERENCE;
#ifdef VCT_PLATFORM_CTR
    s_vctConfig.ecMode          = VCT_EC_STANDARD;
#endif
    s_vctConfig.cid             = static_cast<u16>(ConvertID_Pia2VCT( s_MyCID & 0xFFFF ));
    s_vctConfig.codec           = VCT_CODEC_4BIT_ADPCM;
    s_vctConfig.frameLength     = VCT_AUDIO_FRAME_LENGTH_DEFAULT;
    s_vctConfig.flag            = VCT_CONFIG_FLAG_DEFAULT;
    s_vctConfig.audioBuffer     = s_audioBuffer;
    s_vctConfig.audioBufferSize = VCT_DEFAULT_AUDIO_BUFFER_SIZE;
    s_vctConfig.callback        = EventCallback;
    s_vctConfig.sendCallback    = SendDataCallBack;
    s_vctConfig.userData        = NULL;

    if ( VCT_Init( &s_vctConfig ) == false )
    {
        SAMPLE_PRINT( "Error:VCT_Init()\n" );
        return;
    }
    SAMPLE_PRINT("VCTID=%d\n",s_vctConfig.cid);

    // Display...
    framework->BeginFrame();
    dispConference( diplay_number, pGlobal, vpos );
    framework->EndFrame();

    SAMPLE_PRINT("MemorySize : %d\n",nn::pia::common::GetMemoryUsage());

    while ( true )
    {
#ifdef VCT_PLATFORM_CTR
        if( nn::applet::IsExpectedToProcessHomeButton() ){
            pGlobal->system.sndmicDev->Pause( );
            p_SndMicDevice = NULL;
            nn::applet::ProcessHomeButtonAndWait();
            nn::applet::ClearHomeButtonState();
            nngxUpdateState(NN_GX_STATE_ALL);
            nngxValidateState(NN_GX_STATE_ALL,GL_TRUE);
            VCT_ClearStreaming(  s_vctConfig.session );
            pGlobal->system.sndmicDev->Pause( );
        }
#else
        if(OSGetCoreId() != OSGetMainCoreId())
        {
            ProcUISubProcessMessages(TRUE);
        }
        //Process for system messages.
        ProcUIStatus status = ProcUIProcessMessages(TRUE);
        if (status == PROCUI_STATUS_EXIT)
        {
            //The process needs to be ended, but do not do anything here.
        }
        else if (status == PROCUI_STATUS_RELEASING)
        {
            //The HOME Button has been pressed. Release this process from the foregound and move it to the background.
            pGlobal->system.sndmicDev->Pause( NULL );   //Pause the sound and microphone.
            p_SndMicDevice = NULL;
            SAMPLE_PRINT("to background\n\n");
            ProcUIDrawDoneRelease();
        }
        if(gHomeMenuStatus)   // During the HOME Menu
        {
            if(gHomeMenuStatus == 2)    // If 2, move from the background to the foreground.
            {
                SND_PARAM       sndConfig;
                SndMicDevice    *sndmic   = pGlobal->system.sndmicDev;
                // The sound set is 8 kHz.
                sndConfig.freq         = SND_FREQ_08KHZ;
                sndConfig.frameSample  = VCT_GetAudioFrameLength();
                sndmic->Pause( &sndConfig );    //Resume the paused sound and microphone.
                s_BGMFlg       = false;
                p_SndMicDevice = sndmic;
                SAMPLE_PRINT("restart\n\n");
                VCT_ClearStreaming(  s_vctConfig.session );
                gHomeMenuStatus = 0;    //Foregound.
            }
            continue;
        }
#endif
#ifdef ENABLE_PROFILING
        if(Profiler::GetInstance()->IsCounting() == false){ //Not currently counting.
            // Start the profiler measurement.
            Profiler::GetInstance()->StartProfilingBase(CPU_PROFILING_PERIOD);
        }
#endif

        drawflag = 0;
        //Skip the noise caused by button clicking.
        {
            //Compare the current and prior button states.
            s_OldTrig = s_NowTrig;
            s_NowTrig = framework->IsHold(0xFFFFFFFF);
            if(s_OldTrig != s_NowTrig)
            {
                //The button has been pressed or released. Skip the sending of audio for the specified duration of time.
                VCT_SignalNoiseSkip();
            }
        }
        //Save the current and prior VAD states.
        s_VADOld = s_VADValue;
        s_VADValue = (VCT_GetVADActivity())?1:0;
#if 0 // For Profile...
        nnprofRecordTopMainLoop();
#endif

#ifdef VCT_PLATFORM_CTR
        framework->BeginFrameNoRender();
#else
        framework->BeginFrame();
#endif

        // Cursor Move...
        if ( framework->IsTrigger(nn::pia::framework::BUTTON_UP) )
        {
            vpos = s_Clip( (vpos - 1), 0, vpos_max_[diplay_number] );
            drawflag = 1;
        }
        else if ( framework->IsTrigger(nn::pia::framework::BUTTON_DOWN) )
        {
            vpos = s_Clip( (vpos + 1), 0, vpos_max_[diplay_number] );
            drawflag = 1;
        }

        if ( diplay_number == 0 )
        {
            // Config Conference...
            if ( framework->IsTrigger(nn::pia::framework::BUTTON_A) && (vpos == 0) )
            {
                diplay_number = 1;
                vpos          = 0;
                drawflag = 1;
            }

            // Start Conference...
            else if ( framework->IsTrigger(nn::pia::framework::BUTTON_A) && (vpos == 1) )
            {
                // Clear the join list.
                for (int i = 0; i < VCT_MAX_CONFERENCE_CLIENT; i++)
                {
                    s_joinConferenceList[i] = 0;
                    s_joinCIDList[i]        = nn::pia::STATION_ID_INVALID;
                }
                s_joinCIDNum = 0;

                // Start Conference...
                startConference( pGlobal );
                diplay_number = 2;
                vpos          = 0;
                drawflag = 1;
            }

            // Return...
            else if ( (framework->IsTrigger(nn::pia::framework::BUTTON_A) && (vpos == vpos_max_[diplay_number])) || (net->GetNetInfo( &new_netInfo ) == false) )
            {
                break;
            }
        }
        else if ( diplay_number == 1 )
        {
            if ( configSetting( vpos ) )
            {
                drawflag = 1;
            }

            // Return from Config Conference...
            if ( framework->IsTrigger(nn::pia::framework::BUTTON_A) && (vpos == vpos_max_[diplay_number]) )
            {
                diplay_number = 0;
                vpos          = 0;
                drawflag = 1;
            }
        }
        else if ( diplay_number == 2 )
        {
            // Stop => Return from Start Conference...
            if ( framework->IsTrigger(nn::pia::framework::BUTTON_A) && (vpos == vpos_max_[diplay_number]) || (net->GetNetInfo( &new_netInfo ) == false) )
            {
                // Stop Conference...
                stopConference( pGlobal );

                diplay_number = 0;
                vpos          = 0;
                drawflag = 1;
                goto _vsync;
            }

            // Change Conference Setting...
            else if ( framework->IsTrigger(nn::pia::framework::BUTTON_A) && (vpos == vpos_max_[diplay_number]-1) )
            {
                // Change Conference Setting...
                diplay_number = 3;
                vpos          = 0;
                drawflag = 1;
                goto _vsync;
            }

            // Change Add/Remove Conference Member Menu...
            else if (framework->IsTrigger(nn::pia::framework::BUTTON_X))
            {
#if 1
                s_addConferencePage = ( s_addConferencePage + 1 ) % 4;
#else
                pGlobal->system.sndmicDev->Pause();
#endif
                drawflag = 1;
            }

#ifdef VCT_PLATFORM_CTR
            else if ( framework->IsTrigger(nn::pia::framework::BUTTON_Y ) && ( p_SndMicDevice != NULL ) )
            {
                if ( s_BGMFlg )
                {
                    s_BGMFlg = false;
                    p_SndMicDevice->StopBGM();
                }
                else
                {
                    s_BGMFlg = true;
                    p_SndMicDevice->StartBGM();
                }
            }
#endif

            // Connection Check...
            bool connection_change_flg = false;
            for ( int i = 0; i < NET_MAX_CONNECT; i++ )
            {
                if ( netInfo->CIDList[i] != new_netInfo.CIDList[i] )
                {
                    netInfo->CIDList[i] = new_netInfo.CIDList[i];
                    connection_change_flg = true;
                }
            }
            if ( netInfo->connectionNum != new_netInfo.connectionNum )
            {
                connection_change_flg = true;
                netInfo->connectionNum = new_netInfo.connectionNum;
            }

            if ( connection_change_flg )
            {
                drawflag = 1;
            }

            // Add/Remove Conference Member...
            if ( addRemoveConferenceMember( vpos, &pGlobal->netInformation ) )
            {
                drawflag = 1;
            }
        }
        else if ( diplay_number == 3 )
        {
            // Return Conference Working...
            if ( framework->IsTrigger(nn::pia::framework::BUTTON_A) && (vpos == vpos_max_[diplay_number]) || (net->GetNetInfo( &new_netInfo ) == false) )
            {
                // Change Conference Setting...
                diplay_number = 2;
                vpos          = 0;
                drawflag = 1;
                goto _vsync;
            }

#ifdef VCT_PLATFORM_CTR
            // BGM On/Off
            else if ( framework->IsTrigger(nn::pia::framework::BUTTON_Y ) && ( p_SndMicDevice != NULL ) )
            {
                if ( s_BGMFlg )
                {
                    s_BGMFlg = false;
                    p_SndMicDevice->StopBGM();
                }
                else
                {
                    s_BGMFlg = true;
                    p_SndMicDevice->StartBGM();
                }
            }
#endif

            // Config Setting...
            if ( configSetting2( vpos ) )
            {
                drawflag = 1;
            }
        }

        if ( (diplay_number == 2) || (diplay_number == 3) )
        {
            // Change Information AID...
            if ( framework->IsTrigger(nn::pia::framework::BUTTON_L ) )
            {
                s_InfoCIDNumber = s_Clip( (s_InfoCIDNumber - 1), 0, netInfo->connectionNum-1);
            }
            else if ( framework->IsTrigger(nn::pia::framework::BUTTON_R ) )
            {
                s_InfoCIDNumber = s_Clip( (s_InfoCIDNumber + 1), 0, netInfo->connectionNum-1);
            }

            // Display Audio Information...
            if ( ((audio_info_cnt++) % 10) == 0 || s_VADOld != s_VADValue)
            {
                drawflag |= 2;
                audio_info_cnt = 0;
            }

            // VCT Main...

            VCT_Main();

            //SAMPLE_PRINT( "netInfo->connectionNum: %d.\n", netInfo->connectionNum );
            if (netInfo->connectionNum > 8)
            {
                VCT_Main();
            }
          
        }

_vsync:
#ifdef VCT_PLATFORM_CTR
        if(drawflag == 0){
            framework->EndFrameNoRender();
        }
        else{
            if(drawflag & 2)
                dispAudioInformation( pGlobal );
            if(drawflag & 1)
                dispConference( diplay_number, pGlobal, vpos );
            framework->EndFrame();
        }
#else
        if(drawflag & 2)
            dispAudioInformation( pGlobal );
        dispConference( diplay_number, pGlobal, vpos );
        framework->EndFrame();
#endif
    }

#ifdef ENABLE_PROFILING
    // Stop Profiler measurement.
    Profiler::GetInstance()->FinishProfiling();

    // Finalize Profile...
    Profiler::DestroyInstance();
#endif

    // Finalize VCT...
    VCT_Cleanup();
}


/*---------------------------------------------------------------------------*
  Name        : dispConference
  Description : Draw the conference menu.
  Arguments   : number  - Number from the menu.
              : vpos    - Vertical position of the cursor.
  Returns     : -
 *---------------------------------------------------------------------------*/
static void dispConference( u32 number, SampleVctGlobal *pGlobal, u32 vpos )
{
    switch ( number )
    {
        case 0:
            dispConference0( vpos );
            break;
        case 1:
            dispConference1( vpos );
            break;
        case 2:
            dispConference2( pGlobal, vpos );
            break;
        case 3:
            dispConference3( vpos );
            break;
        default:
            break;
    }
}


/*---------------------------------------------------------------------------*
  Name        : dispConference0
  Description : Draw the conference start menu.
  Arguments   : vpos    - Vertical position of the cursor.
  Returns     : -
 *---------------------------------------------------------------------------*/
/*
    ----------------------------------------------------
    <<<<         Conference Information...          >>>>
    ----------------------------------------------------
     - Audio
         - Frame Length    : --- [mSec]
         - Codec           : 8BIT_RAW/....
     - Spare Packet        : Enable/Disable
         - Spare Codec     : 8BIT_RAW/....
     - Drop Recover        : Enable/Disable
     - High-Pass Filter    : Enable/Disable
     - Echo Canceller      : Enable/Disable
     - VAD                 : Enable/Disable
         - Active Gain     : --- [dB]
         - Inactive Gain   : --- [dB]
         - Clamp Gain      : --- [dB]
         - Release Time    : --- [cnt]
    ----------------------------------------------------

    ----------------------------------------
    <<<<     Conference Start Menu      >>>>
    ----------------------------------------
    > Config Conference...
      Start Conference...
      Return...
    ----------------------------------------
*/
static void dispConference0( u32 vpos )
{
    nn::pia::framework::Framework* framework = nn::pia::framework::Framework::GetInstance();

//    framework->SetRenderTarget( NN_GX_DISPLAY0 );
    framework->ChangeRenderTarget(true);
    framework->DrawText( 0, 8 *  0, "----------------------------------------------------" );
    framework->DrawText( 0, 8 *  2, "<<<<         Conference Information...          >>>>" );
    framework->DrawText( 0, 8 *  4, "----------------------------------------------------" );
    framework->DrawText( 0, 8 *  5, " - Audio" );
    framework->DrawText( 0, 8 *  6, "    - Frame Length    : %10d [mSec]", VCT_GetAudioFrameLength() );
    framework->DrawText( 0, 8 *  7, "    - Codec           : %s",          codec_str_[VCT_GetCodec()] );
    framework->DrawText( 0, 8 * 9, " - Spare Packet       : %s",          enable_str_[VCT_IsEnableSparePacket()] );
    framework->DrawText( 0, 8 * 10, "    - Spare Codec     : %s",          codec_str_[VCT_GetSpareCodec()] );
#if USE_PRIVATE_VCT_API
    framework->DrawText( 0, 8 * 12, " - Drop Recover       : %s",          enable_str_[VCT_IsEnableDropRecover()] );
    framework->DrawText( 0, 8 * 13, " - High-Pass filter   : %s",          enable_str_[VCT_IsEnableHPF()] );
#else
    framework->DrawText( 0, 8 * 12, " - Drop Recover       : --");
    framework->DrawText( 0, 8 * 13, " - High-Pass filter   : --");
#endif
#ifdef VCT_PLATFORM_CTR
    framework->DrawText( 0, 8 * 14, " - Biquad filter      : %s",          enable_str_[VCT_IsEnableBPF()] );
#if USE_PRIVATE_VCT_API
    framework->DrawText( 0, 8 * 15, " - Echo Canceller     : %s",          enable_str_[VCT_IsEnableEchoCancel()] );
#else
    framework->DrawText( 0, 8 * 15, " - Echo Canceller     : --");
#endif
#else
    framework->DrawText( 0, 8 * 14, " - Biquad filter      : %s",          enable_str_[0] );
    framework->DrawText( 0, 8 * 15, " - Echo Canceller     : %s",          enable_str_[1] );
#endif
    framework->DrawText( 0, 8 * 17, " - VAD                : %s",          enable_str_[VCT_IsEnableVAD()] );
    framework->DrawText( 0, 8 * 18, "    - Active Gain     : %10d [dB]",   VCT_GetVADActiveGain() );
    framework->DrawText( 0, 8 * 19, "    - Inactive Gain   : %10d [dB]",   VCT_GetVADInactiveGain() );
    framework->DrawText( 0, 8 * 20, "    - Clamp Gain      : %10ld [dB]",  VCT_GetVADClampGain() );
    framework->DrawText( 0, 8 * 21, "    - Release Time    : %10ld [cnt]", VCT_GetVADReleaseTime() );
    framework->DrawText( 0, 8 * 23, "   - Noise Skip Time  : %10d",        VCT_GetNoiseSkipTime() );
    framework->DrawText( 0, 8 * 24, "----------------------------------------------------" );
    framework->SwapBuffers();

//    framework->SetRenderTarget( NN_GX_DISPLAY1 );
    framework->ChangeRenderTarget(false);
    framework->DrawText( 0, 8 *  4, "----------------------------------------" );
    framework->DrawText( 0, 8 *  6, "<<<<     Conference Start Menu      >>>>" );
    framework->DrawText( 0, 8 *  8, "----------------------------------------" );
    framework->DrawText( 0, 8 * 10, "  Config Conference..." );
    framework->DrawText( 0, 8 * 12, "  Start Conference..." );
    framework->DrawText( 0, 8 * 14, "  Return..." );
    framework->DrawText( 0, 8 * (10 + 2 * vpos), ">" );
}


/*---------------------------------------------------------------------------*
  Name        : dispConference1
  Description : Draw the conference config menu.
  Arguments   : vpos    - Vertical position of the cursor.
  Returns     : -
 *---------------------------------------------------------------------------*/
/*
    ----------------------------------------
    <<<<       Config Conference        >>>>
    ----------------------------------------
    > Frame Length     : 
      Codec            : 
      Spare Packet     : 
      Spare Codec      : 
      Drop Recover     : 
      High-Pass Filter : 
      Biquad Filter    : 
      Echo Canceller   : 
      VAD              : 
      Active Gain      : 
      Inactive Gain    : 
      Clamp Gain       : 
      Release Time     : 
      Duration         : 
      Threshold Rate   : 
      Sampling Packet  : 
      Return...
    ----------------------------------------
*/
static void dispConference1( u32 vpos )
{
    nn::pia::framework::Framework* framework = nn::pia::framework::Framework::GetInstance();

//    framework->SetRenderTarget( NN_GX_DISPLAY0 );
    framework->ChangeRenderTarget(true);
    framework->SwapBuffers();

//    framework->SetRenderTarget( NN_GX_DISPLAY1 );
    framework->ChangeRenderTarget(false);
    framework->DrawText( 0, 8 *  0, "----------------------------------------" );
    framework->DrawText( 0, 8 *  2, "<<<<       Config Conference        >>>>" );
    framework->DrawText( 0, 8 *  4, "----------------------------------------" );
    framework->DrawText( 0, 8 *  5, "  Frame Length     :  %10d",  VCT_GetAudioFrameLength() );
    framework->DrawText( 0, 8 *  6, "  Codec            :  %s",    codec_str_[VCT_GetCodec()] );
    framework->DrawText( 0, 8 *  7, "  Spare Packet     :  %s",    enable_str_[VCT_IsEnableSparePacket()] );
    framework->DrawText( 0, 8 *  8, "  Spare Codec      :  %s",    codec_str_[VCT_GetSpareCodec()] );
#if USE_PRIVATE_VCT_API
    framework->DrawText( 0, 8 *  9, "  Drop Recover     :  %s",    enable_str_[VCT_IsEnableDropRecover()] );
    framework->DrawText( 0, 8 * 10, "  High-Pass filter :  %s",    enable_str_[VCT_IsEnableHPF()] );
#else
    framework->DrawText( 0, 8 *  9, "  Drop Recover     :  --");
    framework->DrawText( 0, 8 * 10, "  High-Pass filter :  --");
#endif
#ifdef VCT_PLATFORM_CTR
    framework->DrawText( 0, 8 * 11, "  Biquad filter    :  %s",    enable_str_[VCT_IsEnableBPF()] );
#if USE_PRIVATE_VCT_API
    framework->DrawText( 0, 8 * 12, "  Echo Canceller   :  %s",    enable_str_[VCT_IsEnableEchoCancel()] );
#else
    framework->DrawText( 0, 8 * 12, "  Echo Canceller   :  --");
#endif
#else
    framework->DrawText( 0, 8 * 11, "  Biquad filter    :  %s",    enable_str_[0] );
    framework->DrawText( 0, 8 * 12, "  Echo Canceller   :  %s",    enable_str_[1] );
#endif
    framework->DrawText( 0, 8 * 13, "  VAD              :  %s",    enable_str_[VCT_IsEnableVAD()] );
    framework->DrawText( 0, 8 * 14, "  Active Gain      :  %10d",  VCT_GetVADActiveGain() );
    framework->DrawText( 0, 8 * 15, "  Inactive Gain    :  %10d",  VCT_GetVADInactiveGain() );
    framework->DrawText( 0, 8 * 16, "  Clamp Gain       :  %10ld", VCT_GetVADClampGain() );
    framework->DrawText( 0, 8 * 17, "  Release Time     :  %10ld", VCT_GetVADReleaseTime() );
    framework->DrawText( 0, 8 * 18, "  Noise Skip Time  :  %10d",  VCT_GetNoiseSkipTime() );
    framework->DrawText( 0, 8 * 19, "  Return..." );
    framework->DrawText( 0, 8 * (5 + vpos), ">" );
}


/*---------------------------------------------------------------------------*
  Name        : dispConference2
  Description : Draw the conference add/remove menu.
  Arguments   : *pGlobal - SampleVctGlobal variable.
              : vpos    - Vertical position of the cursor.
  Returns     : -
 *---------------------------------------------------------------------------*/
/*
    ----------------------------------------
    <<<<      Conference Working       >>>>
    ----------------------------------------
      MY ConnectionID(0x----)
      Press Y => BGM On/Off

    > Member[CID:0x----] : PARTICIPATE/LEAVE
      Member[CID:0x----] : PARTICIPATE/LEAVE
      Member[CID:0x----] : PARTICIPATE/LEAVE
      Member[CID:0x----] : PARTICIPATE/LEAVE
      Member[CID:0x----] : PARTICIPATE/LEAVE
      Member[CID:0x----] : PARTICIPATE/LEAVE
      Member[CID:0x----] : PARTICIPATE/LEAVE
      Member[CID:0x----] : PARTICIPATE/LEAVE
      Change Setting...
      Stop Conference...
    ----------------------------------------
*/
static void dispConference2( SampleVctGlobal *pGlobal, u32 vpos )
{
    nn::pia::framework::Framework* framework = nn::pia::framework::Framework::GetInstance();
    NetInformationStructure *netInfo = &pGlobal->netInformation;
    u32 y;

//    framework->SetRenderTarget( NN_GX_DISPLAY1 );
    framework->ChangeRenderTarget(false);
    framework->DrawText( 0,   0, "----------------------------------------" );
    framework->DrawText( 0,  10, "<<<<      Conference working.       >>>>" );
    framework->DrawText( 0,  20, "----------------------------------------" );
    framework->DrawText( 0,  32, "  MY ConnectionID(0x%04lx)",  s_MyCID );
#ifdef VCT_PLATFORM_CTR
    framework->DrawText( 0,  40, "  Press Y => BGM On/Off." );
#endif
    for(y = 0 ; y < NET_MAX_CONNECT ; y++ )
    {
        framework->DrawText( 0,  52+10*y, "  Member[CID:0x%04lx] %s", netInfo->CIDList[y], netjoin_str_[s_joinConferenceList[y]] );
    }
    framework->DrawText( 0, 52+10*y, "  Change Setting..." );
    framework->DrawText( 0, 62+10*y, "  Stop Conference..." );
    framework->DrawText( 0, 10*vpos + 52, ">" );
    framework->SwapBuffers();
}


/*---------------------------------------------------------------------------*
  Name        : dispConference3
  Description : Draw the change conference setting menu.
  Arguments   : vpos    - Vertical position of the cursor.
  Returns     : -
 *---------------------------------------------------------------------------*/
/*
    ----------------------------------------
    <<<<   Change Conference Setting   >>>>
    ----------------------------------------
      Press Y => BGM On/Off
    > Codec            : 
      Spare Packet     : 
      Spare Codec      : 
      Drop Recover     : 
      High-Pass Filter : 
      Echo Canceller   : 
      VAD              : 
      Active Gain      : 
      Inactive Gain    : 
      Clamp Gain       : 
      Release Time     : 
      Duration         : 
      Threshold Rate   : 
      Sampling Packet  : 
      Mic Gain         : 
      Speaker Volume   : 
      BGM Volume       : 
      Return...
    ----------------------------------------
*/
static void dispConference3( u32 vpos )
{
    nn::pia::framework::Framework* framework = nn::pia::framework::Framework::GetInstance();
#ifdef VCT_PLATFORM_CTR
    u8                        micgain = 0;

    if ( p_SndMicDevice != NULL )
    {
        p_SndMicDevice->MicGainGet( &micgain );
    }
#else
    unsigned int              micgain = 0;

    if ( p_SndMicDevice != NULL )
    {
        p_SndMicDevice->GetGainLevelMic( &micgain );
    }
#endif

//    framework->SetRenderTarget( NN_GX_DISPLAY1 );
    framework->ChangeRenderTarget(false);
    framework->DrawText( 0, 8 *  0, "----------------------------------------" );
    framework->DrawText( 0, 8 *  2, "<<<<   Change Conference setting.   >>>>" );
    framework->DrawText( 0, 8 *  4, "----------------------------------------" );
#ifdef VCT_PLATFORM_CTR
    framework->DrawText( 0, 8 *  5, "Press Y => BGM On/Off." );
#endif
    framework->DrawText( 0, 8 *  6, "  Codec            :  %s",    codec_str_[VCT_GetCodec()] );
    framework->DrawText( 0, 8 *  7, "  Spare Packet     :  %s",    enable_str_[VCT_IsEnableSparePacket()] );
    framework->DrawText( 0, 8 *  8, "  Spare Codec      :  %s",    codec_str_[VCT_GetSpareCodec()] );
#if USE_PRIVATE_VCT_API
    framework->DrawText( 0, 8 *  9, "  Drop Recover     :  %s",    enable_str_[VCT_IsEnableDropRecover()] );
    framework->DrawText( 0, 8 * 10, "  High-Pass filter :  %s",    enable_str_[VCT_IsEnableHPF()] );
#else
    framework->DrawText( 0, 8 *  9, "  Drop Recover     :  --");
    framework->DrawText( 0, 8 * 10, "  High-Pass filter :  --");
#endif
#ifdef VCT_PLATFORM_CTR
    framework->DrawText( 0, 8 * 11, "  Biquad filter    :  %s",    enable_str_[VCT_IsEnableBPF()] );
#if USE_PRIVATE_VCT_API
    framework->DrawText( 0, 8 * 12, "  Echo Canceller   :  %s",    enable_str_[VCT_IsEnableEchoCancel()] );
#else
    framework->DrawText( 0, 8 * 12, "  Echo Canceller   :  --");
#endif
#else
    framework->DrawText( 0, 8 * 11, "  Biquad filter    :  %s",    enable_str_[0] );
    framework->DrawText( 0, 8 * 12, "  Echo Canceller   :  %s",    enable_str_[1] );
#endif
    framework->DrawText( 0, 8 * 13, "  VAD              :  %s",    enable_str_[VCT_IsEnableVAD()] );
    framework->DrawText( 0, 8 * 14, "  Active Gain      :  %10d",  VCT_GetVADActiveGain() );
    framework->DrawText( 0, 8 * 15, "  Inactive Gain    :  %10d",  VCT_GetVADInactiveGain() );
    framework->DrawText( 0, 8 * 16, "  Clamp Gain       :  %10ld", VCT_GetVADClampGain() );
    framework->DrawText( 0, 8 * 17, "  Release Time     :  %10ld", VCT_GetVADReleaseTime() );
    framework->DrawText( 0, 8 * 18, "  Mic Gain         :  %10d",  micgain );
#ifdef VCT_PLATFORM_CTR
    framework->DrawText( 0, 8 * 19, "  Speaker Volume   :  %2.1f", s_Volume );
    framework->DrawText( 0, 8 * 20, "  BGM Volume       :  %2.1f", s_BGMVolume );
#else
    framework->DrawText( 0, 8 * 19, "  Speaker Volume   :  %2d", s_Volume );
    framework->DrawText( 0, 8 * 20, "  BGM Volume       :  (not available)");
#endif
    framework->DrawText( 0, 8 * 21, "  Noise Skip Time  :  %10d",  VCT_GetNoiseSkipTime() );
    framework->DrawText( 0, 8 * 22, "  Return..." );
    framework->DrawText( 0, 8 * (6 + vpos), ">" );
    framework->SwapBuffers();
}


/*---------------------------------------------------------------------------*
  Name        : configSetting
  Description : Conference config setting.
  Arguments   : vpos - Vertical position of the cursor.
  Returns     : true - Change the parameter.
              : false - Do not change the parameter.
 *---------------------------------------------------------------------------*/
static bool configSetting( u32 vpos )
{
    nn::pia::framework::Framework* framework = nn::pia::framework::Framework::GetInstance();

    switch ( vpos )
    {
        case  0:// Frame Length
            if ( framework->IsTrigger(nn::pia::framework::BUTTON_LEFT) )
            {
                VCT_SetAudioFrameLength( s_Clip(VCT_GetAudioFrameLength() - 8, VCT_AUDIO_FRAME_LENGTH_MIN, VCT_AUDIO_FRAME_LENGTH_MAX) );
            }
            else if ( framework->IsTrigger(nn::pia::framework::BUTTON_RIGHT) )
            {
                VCT_SetAudioFrameLength( s_Clip(VCT_GetAudioFrameLength() + 8, VCT_AUDIO_FRAME_LENGTH_MIN, VCT_AUDIO_FRAME_LENGTH_MAX) );
            }
            break;
        case  1:// Codec
#if 0 // Currently, only 4-bit ADPC.
            if ( framework->IsTrigger(nn::pia::framework::BUTTON_LEFT) )
            {
                VCT_SetCodec(VCT_CODEC_2BIT_ADPCM);
            }
            else if ( framework->IsTrigger(nn::pia::framework::BUTTON_RIGHT) )
            {
                VCT_SetCodec(VCT_CODEC_4BIT_ADPCM);
            }
#endif
            break;
        case  2:// Spare Packet
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT) )
            {
                VCT_EnableSparePacket( false );
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT) )
            {
                VCT_EnableSparePacket( true );
            }
            break;
        case  3:// Spare Codec
#if 0 // Currently, only 2-bit ADPC.
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT) )
            {
                VCT_SetSpareCodec(VCT_CODEC_2BIT_ADPCM);
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT) )
            {
                VCT_SetSpareCodec(VCT_CODEC_4BIT_ADPCM);
            }
#endif
            break;
        case  4:// Drop Recover
            if ( framework->IsTrigger(  nn::pia::framework::BUTTON_LEFT) )
            {
#if USE_PRIVATE_VCT_API
                VCT_EnableDropRecover( false );
#endif
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT) )
            {
#if USE_PRIVATE_VCT_API
                VCT_EnableDropRecover( true );
#endif
            }
            break;
        case  5:// High-Pass Filter
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT) )
            {
#if USE_PRIVATE_VCT_API
                VCT_EnableHPF( false );
#endif
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT) )
            {
#if USE_PRIVATE_VCT_API
                VCT_EnableHPF( true );
#endif
            }
            break;
        case  6:// Biquad Filter
#ifdef VCT_PLATFORM_CTR
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT) )
            {
                VCT_EnableBPF( false );
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT) )
            {
                VCT_EnableBPF( true );
            }
#endif
            break;
        case  7:// Echo Canceller
#ifdef VCT_PLATFORM_CTR
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT) )
            {
#if USE_PRIVATE_VCT_API
                VCT_EnableEchoCancel( false );
#endif
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT) )
            {
#if USE_PRIVATE_VCT_API
                VCT_EnableEchoCancel( true );
#endif
            }
#endif
            break;
        case  8:// VAD
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT) )
            {
                VCT_EnableVAD( false );
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT) )
            {
                VCT_EnableVAD( true );
            }
            break;
        case  9:// Active Gain
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT) )
            {
                VCT_SetVADActiveGain( s_Clip(VCT_GetVADActiveGain() - 1, 0, 1000) );
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT) )
            {
                VCT_SetVADActiveGain( s_Clip(VCT_GetVADActiveGain() + 1, 0, 1000) );
            }
            break;
        case  10:// Inactive Gain
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT) )
            {
                VCT_SetVADInactiveGain( s_Clip(VCT_GetVADInactiveGain() - 1, 0, 1000) );
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT) )
            {
                VCT_SetVADInactiveGain( s_Clip(VCT_GetVADInactiveGain() + 1, 0, 1000) );
            }
            break;
        case 11:// Clamp Gain
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT) )
            {
                VCT_SetVADClampGain( s_Clip(VCT_GetVADClampGain() - 1, 0, VCT_VAD_CLAMP_GAIN_MAX) );
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT) )
            {
                VCT_SetVADClampGain( s_Clip(VCT_GetVADClampGain() + 1, 0, VCT_VAD_CLAMP_GAIN_MAX) );
            }
            break;
        case 12:// Release Time
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT) )
            {
                VCT_SetVADReleaseTime( s_Clip(VCT_GetVADReleaseTime() - 1, 0, 10000) );
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT) )
            {
                VCT_SetVADReleaseTime( s_Clip(VCT_GetVADReleaseTime() + 1, 0, 10000) );
            }
            break;
        case 13:// Noise Skip Time
            {
            u32 lp = VCT_GetNoiseSkipTime();
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) )
            {
                if(lp > 0){
                    lp -= 100;
                    VCT_SetNoiseSkipTime(lp);
                }
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) )
            {
                if(lp < 1000){
                    lp += 100;
                    VCT_SetNoiseSkipTime(lp);
                }
            }
            }
            break;
        default:
            return false;  // No change.
    }

    return true;
}


/*---------------------------------------------------------------------------*
  Name        : configSetting2
  Description : Conference config setting 2.
  Arguments   : vpos - Vertical position of the cursor.
  Returns     : true - Change the parameter.
              : false - Do not change the parameter.
 *---------------------------------------------------------------------------*/
static bool configSetting2( u32 vpos )
{
    nn::pia::framework::Framework* framework = nn::pia::framework::Framework::GetInstance();
    bool changeFlg = false;

    switch ( vpos )
    {
        case  0:// Codec
#if 0 // Currently, only 4-bit ADPC.
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) )
            {
                VCT_SetCodec(VCT_CODEC_2BIT_ADPCM);
                changeFlg = true;
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) )
            {
                VCT_SetCodec(VCT_CODEC_4BIT_ADPCM);
                changeFlg = true;
            }
#endif
            break;
        case  1:// Spare Packet
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) )
            {
                VCT_EnableSparePacket( false );
                changeFlg = true;
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) )
            {
                VCT_EnableSparePacket( true );
                changeFlg = true;
            }
            break;
        case  2:// Spare Codec
#if 0 // Currently, only 2-bit ADPC.
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) )
            {
                VCT_SetSpareCodec(VCT_CODEC_2BIT_ADPCM);
                changeFlg = true;
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) )
            {
                VCT_SetSpareCodec(VCT_CODEC_4BIT_ADPCM);
                changeFlg = true;
            }
#endif
            break;
        case  3:// Drop Recover
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) )
            {
#if USE_PRIVATE_VCT_API
                VCT_EnableDropRecover( false );
                changeFlg = true;
#endif
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) )
            {
#if USE_PRIVATE_VCT_API
                VCT_EnableDropRecover( true );
                changeFlg = true;
#endif
            }
            break;
        case  4:// High-Pass Filter
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) )
            {
#if USE_PRIVATE_VCT_API
                VCT_EnableHPF( false );
                changeFlg = true;
#endif
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) )
            {
#if USE_PRIVATE_VCT_API
                VCT_EnableHPF( true );
                changeFlg = true;
#endif
            }
            break;
        case  5:// Band-Pass Filter
#ifdef VCT_PLATFORM_CTR
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) )
            {
                VCT_EnableBPF( false );
                changeFlg = true;
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) )
            {
                VCT_EnableBPF( true );
                changeFlg = true;
            }
#endif
            break;
        case  6:// Echo Canceller
#ifdef VCT_PLATFORM_CTR
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) )
            {
#if USE_PRIVATE_VCT_API
                VCT_EnableEchoCancel( false );
                changeFlg = true;
#endif
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) )
            {
#if USE_PRIVATE_VCT_API
                VCT_EnableEchoCancel( true );
                changeFlg = true;
#endif
            }
#endif
            break;
        case  7:// VAD
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) )
            {
                VCT_EnableVAD( false );
                changeFlg = true;
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) )
            {
                VCT_EnableVAD( true );
                changeFlg = true;
            }
            break;
        case  8:// Active Gain
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) )
            {
                VCT_SetVADActiveGain( s_Clip(VCT_GetVADActiveGain() - 1, 0, 1000) );
                changeFlg = true;
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) )
            {
                VCT_SetVADActiveGain( s_Clip(VCT_GetVADActiveGain() + 1, 0, 1000) );
                changeFlg = true;
            }
            break;
        case  9:// Inactive Gain
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) )
            {
                VCT_SetVADInactiveGain( s_Clip(VCT_GetVADInactiveGain() - 1, 0, 1000) );
                changeFlg = true;
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) )
            {
                VCT_SetVADInactiveGain( s_Clip(VCT_GetVADInactiveGain() + 1, 0, 1000) );
                changeFlg = true;
            }
            break;
        case  10:// Clamp Gain
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) )
            {
                VCT_SetVADClampGain( s_Clip(VCT_GetVADClampGain() - 1, 0, VCT_VAD_CLAMP_GAIN_MAX) );
                changeFlg = true;
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) )
            {
                VCT_SetVADClampGain( s_Clip(VCT_GetVADClampGain() + 1, 0, VCT_VAD_CLAMP_GAIN_MAX) );
                changeFlg = true;
            }
            break;
        case 11:// Release Time
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) )
            {
                VCT_SetVADReleaseTime( s_Clip(VCT_GetVADReleaseTime() - 1, 0, 10000) );
                changeFlg = true;
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) )
            {
                VCT_SetVADReleaseTime( s_Clip(VCT_GetVADReleaseTime() + 1, 0, 10000) );
                changeFlg = true;
            }
            break;
        case 12:// MIC Gain
#ifdef VCT_PLATFORM_CTR
            u8 micgain;
            if (( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) ) && ( p_SndMicDevice != NULL ))
            {
                p_SndMicDevice->MicGainGet( &micgain );
                micgain = ( s_Clip(micgain - 1, 31, 67) );
                p_SndMicDevice->MicGainSet( micgain );
                changeFlg = true;
            }
            else if (( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) ) && ( p_SndMicDevice != NULL ))
            {
                p_SndMicDevice->MicGainGet( &micgain );
                micgain = ( s_Clip(micgain + 1, 1, 100) );
                p_SndMicDevice->MicGainSet( micgain );
                changeFlg = true;
            }
#else
            unsigned int micgain;
            if (( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) ) && ( p_SndMicDevice != NULL ))
            {
                p_SndMicDevice->GetGainLevelMic( &micgain );
                if(micgain > s_micGainMin){
                    micgain -= s_micGainStep;
                    p_SndMicDevice->SetGainLevelMic( micgain );
                    changeFlg = true;
                }
            }
            else if (( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) ) && ( p_SndMicDevice != NULL ))
            {
                p_SndMicDevice->GetGainLevelMic( &micgain );
                if(micgain < s_micGainMax){
                    micgain += s_micGainStep;
                    p_SndMicDevice->SetGainLevelMic( micgain );
                    changeFlg = true;
                }
            }
#endif
            break;
        case 13:// Speaker Volume
#ifdef VCT_PLATFORM_CTR
            if (( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) ) && ( p_SndMicDevice != NULL ))
            {
                s_Volume = s_Volume - 0.1f;
                if ( s_Volume < 0.0f )
                {
                    s_Volume = 0.0f;
                }
                p_SndMicDevice->SetVolume( s_Volume );
                changeFlg = true;
            }
            else if (( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) ) && ( p_SndMicDevice != NULL ))
            {
                s_Volume = s_Volume + 0.1f;
                if ( s_Volume > 20.0f )
                {
                    s_Volume = 20.0f;
                }
                p_SndMicDevice->SetVolume( s_Volume );
                changeFlg = true;
            }
#else
            if (( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) ) && ( p_SndMicDevice != NULL ))
            {
                if ( s_Volume > 0 )
                {
                    s_Volume--;
                    p_SndMicDevice->SetDRCVolume( s_Volume );
                    changeFlg = true;
                }
            }
            else if (( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) ) && ( p_SndMicDevice != NULL ))
            {
                if ( s_Volume < (SPEAKER_GAIN_IDX_MAX - 1) )
                {
                    s_Volume++;
                    p_SndMicDevice->SetDRCVolume( s_Volume );
                    changeFlg = true;
                }
            }
#endif
            break;
        case 14:// BGM Volume
#ifdef VCT_PLATFORM_CTR
            if (( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) ) && ( p_SndMicDevice != NULL ))
            {
                s_BGMVolume = s_BGMVolume - 0.1f;
                if ( s_BGMVolume < 0.0f )
                {
                    s_BGMVolume = 0.0f;
                }
                p_SndMicDevice->SetBGMVolume( s_BGMVolume );
                changeFlg = true;
            }
            else if (( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) ) && ( p_SndMicDevice != NULL ))
            {
                s_BGMVolume = s_BGMVolume + 0.1f;
                if ( s_BGMVolume > 10.0f )
                {
                    s_BGMVolume = 10.0f;
                }
                p_SndMicDevice->SetBGMVolume( s_BGMVolume );
                changeFlg = true;
            }
#endif
            break;
        case 15:// Noise Skip Time
            {
            u32 lp = VCT_GetNoiseSkipTime();
            if ( framework->IsTrigger( nn::pia::framework::BUTTON_LEFT ) )
            {
                if(lp > 0){
                    lp -= 100;
                    VCT_SetNoiseSkipTime(lp);
                    changeFlg = true;
                }
            }
            else if ( framework->IsTrigger( nn::pia::framework::BUTTON_RIGHT ) )
            {
                if(lp < 1000){
                    lp += 100;
                    VCT_SetNoiseSkipTime(lp);
                    changeFlg = true;
                }
            }
            }
            break;
        default:
            break;
    }

    return changeFlg;
}


/*---------------------------------------------------------------------------*
  Name        : addRemoveConferenceMember
  Description : Add or remove a conference member.
  Arguments   : *pad - Pad reader object.
              : vpos - Vertical position of the cursor.
              : *netInfo - Network information.
  Returns     : true - Change the parameter.
              : false - Do not change the parameter.
 *---------------------------------------------------------------------------*/
static bool addRemoveConferenceMember( u32 vpos, NetInformationStructure *netInfo )
{
    nn::pia::framework::Framework* framework = nn::pia::framework::Framework::GetInstance();
    bool ret = false;

    // Check User Request...
    if ( /*(vpos >= 0) &&*/ (vpos <= (NET_MAX_CONNECT-1)) && framework->IsTrigger( nn::pia::framework::BUTTON_A) && (netInfo->CIDList[vpos] != s_MyCID) && (netInfo->CIDList[vpos] != nn::pia::STATION_ID_INVALID) )
    {
        int  i;
        bool addFlg = true;
        for ( i = 0; i < s_joinCIDNum; i++ )
        {
            if ( s_joinCIDList[i] == netInfo->CIDList[vpos] )
            {
                addFlg = false;
                break;
            }
        }

        if ( addFlg )
        {
            // Add Member...
            SAMPLE_PRINT("Add member(%ld)\n", netInfo->CIDList[vpos]);
            s_joinCIDList[s_joinCIDNum++] = netInfo->CIDList[vpos];
            if ( VCT_AddConferenceClient( ConvertID_Pia2VCT(netInfo->CIDList[vpos]) ) != VCT_ERROR_NONE )
            {
                SAMPLE_PRINT( "Fail Add Client (%ld).\n", netInfo->CIDList[vpos] );
            }
        }
        else
        {
            // Remove and Sort List...
            SAMPLE_PRINT("Remove member(%ld)\n", s_joinCIDList[i]);
            if ( VCT_RemoveConferenceClient( ConvertID_Pia2VCT(s_joinCIDList[i]) ) != VCT_ERROR_NONE )
            {
                SAMPLE_PRINT( "Fail Remove Client (%ld).\n", s_joinCIDList[i] );
            }
            for ( ; i < (VCT_MAX_CONFERENCE_CLIENT - 1); i++ )
            {
                s_joinCIDList[i]   = s_joinCIDList[i+1];
                s_joinCIDList[i+1] = nn::pia::STATION_ID_INVALID;
            }
            s_joinCIDNum--;
        }
        ret = true;
    }

    // Check Leave...
    for ( int i = 0; i < s_joinCIDNum; i++ )
    {
        bool leaveFlg = true;
#if 0
        for ( int j = 0; j < netInfo->connectionNum; j++ )
#else
        //n1769 CIDList may still not be ready at this point, so we must adjust all of the elements.
        //      In the original code above, one station in a session leaving was enough to make the other stations appear to leave the session also.
        //      
        for ( int j = 0; j < NET_MAX_CONNECT; j++ )
#endif
        {
            if ( s_joinCIDList[i] == netInfo->CIDList[j] )
            {
                leaveFlg = false;
                break;
            }
        }

        if ( leaveFlg )
        {
            if ( VCT_RemoveConferenceClient( ConvertID_Pia2VCT(s_joinCIDList[i]) ) != VCT_ERROR_NONE )
            {
                SAMPLE_PRINT( "Fail Remove Client (%ld).\n", s_joinCIDList[i] );
            }
            for ( int j = i; j < (VCT_MAX_CONFERENCE_CLIENT - 1); j++ )
            {
                s_joinCIDList[j]   = s_joinCIDList[j+1];
                s_joinCIDList[j+1] = nn::pia::STATION_ID_INVALID;
            }
            s_joinCIDNum--;
            ret = true;
            break;
        }
    }

    for ( int i = 0; i < VCT_MAX_CONFERENCE_CLIENT; i++ )
    {
        s_joinConferenceList[i] = 0;

        for ( int j = 0; j < s_joinCIDNum; j++ )
        {
            if ( netInfo->CIDList[i] == s_joinCIDList[j] )
            {
                s_joinConferenceList[i] = 1;
            }
        }
    }

    return ret;
}


/*---------------------------------------------------------------------------*
  Name        : startConference
  Description : Start conference.
  Arguments   : *pGlobal - Render system object.
  Returns     : -
 *---------------------------------------------------------------------------*/
static void startConference( SampleVctGlobal *pGlobal )
{
    SndMicDevice    *sndmic   = pGlobal->system.sndmicDev;

#ifdef VCT_PLATFORM_CTR
    SndMicConfigStructure   sndmicConfig;

    // Set Snd/Mic Config...
    sndmicConfig.frame_msec = VCT_GetAudioFrameLength();
    sndmicConfig.mic_gain   = 60;

    // Initialize Snd/Mic...
    if ( sndmic->Initialize( &sndmicConfig ) != SNDMIC_ERROR_NONE )
    {
        SAMPLE_PRINT( "Error:Initialize snd or mic.\n" );
    }

    // Start Snd/Mic...
    if ( sndmic->Start() != SNDMIC_ERROR_NONE )
    {
        SAMPLE_PRINT( "Error:Start snd or mic.\n" );
    }
#else
    SND_PARAM       sndConfig;

    // The sound set is 8 kHz.
    sndConfig.freq         = SND_FREQ_08KHZ;
    sndConfig.frameSample      = VCT_GetAudioFrameLength();

    sndmic->InitializeMicSndCommon( &sndConfig );

    MIC_STATE_INFO micStateInfo;
    sndmic->GetStateMic( &micStateInfo );
    s_micGainMax  = micStateInfo.micGainMax;
    s_micGainMin  = micStateInfo.micGainMin;
    s_micGainStep = micStateInfo.micGainStep;
#endif

    s_BGMFlg       = false;
    p_SndMicDevice = sndmic;
}


/*---------------------------------------------------------------------------*
  Name        : stopConference
  Description : Stop the conference.
  Arguments   : *pGlobal - Render system object.
  Returns     : -
 *---------------------------------------------------------------------------*/
static void stopConference( SampleVctGlobal *pGlobal )
{
    SndMicDevice *sndmic = pGlobal->system.sndmicDev;

    // Remove Added Client...
    for ( int i = 0; i < VCT_MAX_CONFERENCE_CLIENT; i++ )
    {
        if ( s_joinConferenceList[i] == 1 )
        {
            s_joinConferenceList[i] = 0;
            if ( VCT_RemoveConferenceClient( ConvertID_Pia2VCT(pGlobal->netInformation.CIDList[i]) ) != VCT_ERROR_NONE )
            {
                SAMPLE_PRINT( "Fail Remove Client (%d).\n", pGlobal->netInformation.CIDList[i] );
            }
        }
    }

#ifdef VCT_PLATFORM_CTR
    if ( s_BGMFlg )
    {
        s_BGMFlg = false;
        sndmic->StopBGM();
    }

    // Stop Snd/Mic
    if ( sndmic->Stop() != SNDMIC_ERROR_NONE )
    {
        SAMPLE_PRINT( "Error:Stop snd or mic.\n" );
    }

    // Finalize Snd/Mic
    if ( sndmic->Finalize() != SNDMIC_ERROR_NONE )
    {
        SAMPLE_PRINT( "Error:Finalize snd or mic.\n" );
    }
#else
    // Stop Snd/Mic
    if ( sndmic->StopMic() != SNDMIC_ERROR_NONE )
    {
        SAMPLE_PRINT( "Error:Stop mic.\n" );
    }
    if ( sndmic->StopSnd() != SNDMIC_ERROR_NONE )
    {
        SAMPLE_PRINT( "Error:Stop snd.\n" );
    }

    // Finalize Snd/Mic
    if ( sndmic->FinalizeMic() != SNDMIC_ERROR_NONE )
    {
        SAMPLE_PRINT( "Error:Finalize mic.\n" );
    }
    if ( sndmic->FinalizeSnd() != SNDMIC_ERROR_NONE )
    {
        SAMPLE_PRINT( "Error:Finalize snd.\n" );
    }

    sndmic->FinalizeMicSndCommon();
#endif

    p_SndMicDevice = NULL;
}


/*---------------------------------------------------------------------------*
  Name        : dispAudioInformation
  Description : Display audio information.
  Arguments   : *pGlobal - Render system object.
  Returns     : -
 *---------------------------------------------------------------------------*/
/*
    ----------------------------------------------------
    <<<<          Audio Information...              >>>>
    ----------------------------------------------------
     ConnectionID              : 0x----
     Sequence                  : ---
     Internal Latency          : ---
     Buffer Latency            : ---
     Jitter                    : ---
     Clock Skew                : ---
     Buffer Count              : ---
     Drop Count                : ---
     Continuous Drop           : ---
     Jam Count                 : ---
     Duplicate Count           : ---
     Wait Count                : ---
     Late Count                : ---
     Recent Late Count         : ---
     Recent Receive Count      : ---
     Recent Late Ratio         : ---
     Recover Count             : ---
     Recover Using Spare Count : ---


     CPU Performance           : --- [%]
    ----------------------------------------------------
*/
static void dispAudioInformation( SampleVctGlobal *pGlobal )
{
    VCTAudioInfo audioInfo;
    nn::pia::framework::Framework* framework = nn::pia::framework::Framework::GetInstance();

    // Get Audio Info...
    VCT_GetAudioInfoFromCid( ConvertID_Pia2VCT(pGlobal->netInformation.CIDList[s_InfoCIDNumber]), &audioInfo );

//    framework->SetRenderTarget( NN_GX_DISPLAY0 );
    framework->ChangeRenderTarget(true);
    framework->DrawText( 0, 8 *  0, "----------------------------------------------------" );
    framework->DrawText( 0, 8 *  2, "<<<<          Audio Information...              >>>>" );
    framework->DrawText( 0, 8 *  4, "----------------------------------------------------" );
    framework->DrawText( 0, 8 *  6, " ConnectionID              : 0x%04lx",   pGlobal->netInformation.CIDList[s_InfoCIDNumber] );
    framework->DrawText( 0, 8 *  7, " Sequence                  : %ld",       audioInfo.sequence );
    framework->DrawText( 0, 8 *  8, " Internal Latency          : %ld",       audioInfo.internalLatency );
#ifdef VCT_PLATFORM_CTR
    framework->DrawText( 0, 8 *  9, " Buffer Latency            : %ld",       static_cast<u32>(audioInfo.bufferLatency.ToTimeSpan().GetMilliSeconds()) );
    framework->DrawText( 0, 8 * 10, " Jitter                    : %ld",       static_cast<u32>(audioInfo.jitter.ToTimeSpan().GetMilliSeconds()) );
    framework->DrawText( 0, 8 * 11, " Clock Skew                : %ld",       static_cast<u32>(audioInfo.clockSkew.ToTimeSpan().GetMilliSeconds()) );
#else
    framework->DrawText( 0, 8 *  9, " Buffer Latency            : %ld",       static_cast<u32>(OSTicksToMilliseconds(audioInfo.bufferLatency)) );
    framework->DrawText( 0, 8 * 10, " Jitter                    : %ld",       static_cast<u32>(OSTicksToMilliseconds(audioInfo.jitter)) );
    framework->DrawText( 0, 8 * 11, " Clock Skew                : %ld",       static_cast<u32>(OSTicksToMilliseconds(audioInfo.clockSkew)) );
#endif
    framework->DrawText( 0, 8 * 12, " Buffer Count              : %ld",       audioInfo.bufferCount );
    framework->DrawText( 0, 8 * 13, " Drop Count                : %ld",       audioInfo.dropCount );
    framework->DrawText( 0, 8 * 14, " Continuous Drop           : %ld",       audioInfo.continuousDrop );
    framework->DrawText( 0, 8 * 15, " Jam Count                 : %ld",       audioInfo.jamCount );
    framework->DrawText( 0, 8 * 16, " Duplicate Count           : %ld",       audioInfo.duplicateCount );
    framework->DrawText( 0, 8 * 17, " Wait Count                : %ld",       audioInfo.waitCount );
    framework->DrawText( 0, 8 * 18, " Late Count                : %ld",       audioInfo.lateCount );
    framework->DrawText( 0, 8 * 19, " Recent Late Count         : %ld",       audioInfo.recentLateCount );
    framework->DrawText( 0, 8 * 20, " Recent Receive Count      : %ld",       audioInfo.recentReceiveCount );
    framework->DrawText( 0, 8 * 21, " Recent Late Ratio         : %ld",       audioInfo.recentLateRatio );
    framework->DrawText( 0, 8 * 22, " Recover Count             : %ld",       audioInfo.recoverCount );
    framework->DrawText( 0, 8 * 23, " Recover Using Spare Count : %ld",       audioInfo.recoverUsingSpareCount );
#ifdef ENABLE_PROFILING
//    framework->DrawText( 0, 8 * 24, " CPU Performance           : %ld",       loadPercentage );
#endif
    framework->DrawText( 0, 8 * 25, " VAD Value                 : %d",       s_VADValue );
    framework->DrawText( 0, 8 * 27, "----------------------------------------------------" );
    framework->SwapBuffers();
}


/*---------------------------------------------------------------------------*
  Name        : EventCallback
  Description : Event callback function.
  Arguments   : cid      - src connectionID.
              : result   - Event result.
              : *session - Session.
              : *param   - Parameter.
  Returns     : -
 *---------------------------------------------------------------------------*/
static void EventCallback( u16 cid, VCTEvent result, VCTSession *session, void *param )
{
    (void)param;

    switch ( result )
    {
        case VCT_EVENT_DISCONNECTED:
            SAMPLE_PRINT( "VCT disconnected event(%d).\n", ConvertID_VCT2Pia(cid) );
            VCT_StopStreaming( session );
            if ( VCT_DeleteSession( session ) == false )
            {
                SAMPLE_PRINT( "fail delete session.\n" );
            }
            break;
        case VCT_EVENT_CONNECTED:
            SAMPLE_PRINT( "VCT connected event(%d).\n", ConvertID_VCT2Pia(cid) );
            if ( VCT_StartStreaming( session ) == false )
            {
                SAMPLE_PRINT( "fail start streming.\n" );
            }
            break;
        default:
            break;
    }
}


/*---------------------------------------------------------------------------*
  Name        : SendDataCallBack
  Description : Send data callback function.
  Arguments   : cid        - Destination ConnectionId.
              : *buff      - Send data.
              : size       - Send data size.
              : isReliable - (true)reliable, (false)unreliable
  Returns     : true  - Success.
              : false - Failure.
 *---------------------------------------------------------------------------*/
//#ifdef VCT__CTR
static bool SendDataCallBack( u16 cid, const void *buff, u32 size, bool isReliable )
//#else
//static BOOL SendDataCallBack( u16 cid, const void *buff, u32 size, BOOL isReliable )
//#endif
{
    return ( p_NetInfra->ReqSendData(ConvertID_VCT2Pia(static_cast<u32>(cid)), (void *)buff, size, isReliable) );
}

/* <<<< EOF >>>> */
