/*---------------------------------------------------------------------------*
Project:  Voice Chat Sample.
File:     game_network.cpp

Copyright (C) 2011-2014 Nintendo.  All rights reserved.

These coded instructions, statements, and computer programs contain proprietary
information of Nintendo and/or its licensed developers and are protected by
national and international copyright laws. They may not be disclosed to third
parties or copied or duplicated in any form, in whole or in part, without the
prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*
    Include
 *---------------------------------------------------------------------------*/
#ifdef VCT_PLATFORM_CTR
#include <nn.h>
#include <nn/cfg.h>
#else
#include <cafe.h>
#endif

#include "framework.h"

#include "game_network.h"
#include "game_conference.h"
#include "net.h"
#include "utilities.h"


/*---------------------------------------------------------------------------*
    Type
 *---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*
    External
 *---------------------------------------------------------------------------*/
extern u32 ConvertID_Pia2VCT(u32);  //net.cpp
extern u32 ConvertID_VCT2Pia(u32);  //net.cpp


/*---------------------------------------------------------------------------*
    Prototype
 *---------------------------------------------------------------------------*/
static void receiveCallback( u32 srcCID, void *buffer, u32 size );
static void dispNetwork0( SampleVctGlobal *pGlobal, u32 vpos );
static void dispNetwork1( u32 vpos );
static void dispNetwork2( SampleVctGlobal *pGlobal, u32 vpos );
static void dispNetwork3( u32 vpos );


/*---------------------------------------------------------------------------*
    Function
 *---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*
  Name        : NetworkStart
  Description : Start network.
  Arguments   : *pGlobal - SampleVctGlobal variable.
  Returns     : -
 *---------------------------------------------------------------------------*/
int NetworkStart( SampleVctGlobal *pGlobal )
{
    nn::pia::framework::Framework* pFw = nn::pia::framework::Framework::GetInstance();
    NetworkInfra              *netInfra  = pGlobal->system.netInfra;
    SampleVctNetParam         *netParam  = &pGlobal->netParameter;
    NetConfigStructure        netConfig;
    NetInformationStructure   netInfo;
    u32                       drawflag;
    u32                       vpos       = 0;
    int                       checkflag = 0;
    bool                      connecting = false;

    // Display...
#ifdef VCT_PLATFORM_CTR
    pFw->BeginFrame();
    dispNetwork0( pGlobal, vpos );
    pFw->EndFrame();
#else
    drawflag = 0;
#endif

    while ( checkflag == 0 )
    {
#ifdef VCT_PLATFORM_CTR
        drawflag = 0;
        pFw->BeginFrameNoRender();
#else
        pFw->BeginFrame();
#endif

        if ( connecting == false)
        {
            // Cursor move...
            if ( pFw->IsTrigger(nn::pia::framework::BUTTON_UP))
            {
                vpos = s_Clip( (vpos - 1), 0, 3 );
#ifdef VCT_PLATFORM_CTR
                dispNetwork0( pGlobal, vpos );
                drawflag = 1;
#else
                drawflag = 0;
#endif
            }
            else if ( pFw->IsTrigger(nn::pia::framework::BUTTON_DOWN))
            {
                vpos = s_Clip( (vpos + 1), 0, 3 );
#ifdef VCT_PLATFORM_CTR
                dispNetwork0( pGlobal, vpos );
                drawflag = 1;
#else
                drawflag = 0;
#endif
            }

            // Parameter change...
            else if ( pFw->IsTrigger(nn::pia::framework::BUTTON_LEFT))
            {
                if ( vpos == 0 )
                {
                    netParam->netMatchMode = SAMPLE_VCT_NET_MATCH_ANYBODY;
                }
                else if ( vpos == 1 )
                {
                    netParam->gameMode = s_Clip( (netParam->gameMode - 1), 1, 2000 );
                }
                else if ( vpos == 2 )
                {
                    netParam->maxConnect = s_Clip( (netParam->maxConnect - 1), 1, 16 );
                }
#ifdef VCT_PLATFORM_CTR
                dispNetwork0( pGlobal, vpos );
                drawflag = 1;
#else
                drawflag = 0;
#endif
            }
            else if ( pFw->IsTrigger(nn::pia::framework::BUTTON_RIGHT))
            {
                if ( vpos == 0 )
                {
                    netParam->netMatchMode = SAMPLE_VCT_NET_MATCH_FRIEND;
                }
                else if ( vpos == 1 )
                {
                    netParam->gameMode = s_Clip( (netParam->gameMode + 1), 1, 2000 );
                }
                else if ( vpos == 2 )
                {
                    netParam->maxConnect = s_Clip( (netParam->maxConnect + 1), 1, 16 );
                }
#ifdef VCT_PLATFORM_CTR
                dispNetwork0( pGlobal, vpos );
                drawflag = 1;
#else
                drawflag = 0;
#endif
            }

            // Start Network...
            else if ( pFw->IsTrigger(nn::pia::framework::BUTTON_A) && (vpos == 3) )
            {
                connecting = true;
#ifdef VCT_PLATFORM_CTR
                dispNetwork1( vpos );
                drawflag = 1;
#else
                drawflag = 1;
#endif

                // Network start.
                netConfig.eventCallbackFunction  = NULL;
                netConfig.receiveCallbackFunction = receiveCallback;
                netConfig.gameMode             = netParam->gameMode;
                netConfig.maxConnect           = netParam->maxConnect;
                if ( netParam->netMatchMode == SAMPLE_VCT_NET_MATCH_ANYBODY )
                {
                    netConfig.netMatchMode   = NET_MATCH_ANYBODY;
                }
                else
                {
                    netConfig.netMatchMode   = NET_MATCH_FRIEND;
                }
                netInfra->Start( &netConfig );
            }
        }
        else if ( netInfra->GetNetInfo( &netInfo ) )
        {
            // Copy network information...
            pGlobal->netInformation.hostAddr        = netInfo.hostAddr;
            pGlobal->netInformation.netMask         = netInfo.netMask;
            pGlobal->netInformation.dns1            = netInfo.dns1;
            pGlobal->netInformation.dns2            = netInfo.dns2;
            pGlobal->netInformation.gateway         = netInfo.gateway;
            for ( int i = 0; i < NET_MAX_CONNECT; i ++ )
            {
                pGlobal->netInformation.CIDList[i] = netInfo.CIDList[i];
            }
            pGlobal->netInformation.connectionNum = netInfo.connectionNum;
            pGlobal->netInformation.ownerPid      = netInfo.ownerPid;
            pGlobal->netInformation.myPid         = netInfo.myPid;
            pGlobal->netInformation.myCid         = netInfo.myCid;
#ifdef VCT_PLATFORM_CTR
            std::memcpy( pGlobal->netInformation.mac, netInfo.mac, sizeof(u8)*nn::socket::ETH_ALEN );
#else
            //Cafe does not allow acquisition of MAC address.
#endif
            checkflag = 1;
        }
        else if ( netInfra->CheckNetError() )
        {
            netInfra->Stop();
            connecting = false;
            //checkflag = (-1);
#ifdef VCT_PLATFORM_CTR
#else
            drawflag = 0;
#endif
        }

#ifdef VCT_PLATFORM_CTR
        if(drawflag == 0){
            pFw->EndFrameNoRender();
        }
        else{
            pFw->EndFrame();
        }
#else
        if(drawflag == 0)
        {
            dispNetwork0( pGlobal, vpos );
        }
        else if(drawflag == 1)
        {
            dispNetwork1( vpos );
        }
        pFw->EndFrame();
#endif
    }

    return checkflag;
}


/*---------------------------------------------------------------------------*
  Name        : NetworkConnected
  Description : network connected.
  Arguments   : *pGlobal - SampleVctGlobal variable.
  Returns     : -
 *---------------------------------------------------------------------------*/
void NetworkConnected( SampleVctGlobal *pGlobal )
{
    nn::pia::framework::Framework* pFw = nn::pia::framework::Framework::GetInstance();
    NetworkInfra              *netInfra  = pGlobal->system.netInfra;
    u32                       vpos       = 0;
    u32                       drawflag;
    bool                      changemode;
    bool                      loopflag = true;
    bool                      stopflag = false;
#ifdef VCT_PLATFORM_CTR
#else
    drawflag = 2;
#endif

    while( loopflag ){
        changemode = false;
        // Display...
#ifdef VCT_PLATFORM_CTR
        pFw->BeginFrame();
        dispNetwork2( pGlobal, vpos );
        pFw->EndFrame();
#else
        drawflag = 2;
#endif

        while(changemode == false){
            changemode = false;
#ifdef VCT_PLATFORM_CTR
            drawflag = 0;
            pFw->BeginFrameNoRender();
#else
            pFw->BeginFrame();
#endif
            if ( pFw->IsTrigger(nn::pia::framework::BUTTON_A))
            {
                if(vpos < 1)    //Conference
                {
                    changemode = true;
                }
                else{
#ifdef VCT_PLATFORM_CTR
                    dispNetwork3( vpos );
                    drawflag = 1;
#else
                    drawflag = 3;
#endif
                    // Network stop.
                    netInfra->Stop();
                    stopflag = true;
                }
            }
            else if ( pFw->IsTrigger(nn::pia::framework::BUTTON_UP))
            {
                vpos = s_Clip( (vpos - 1), 0, 1 );
#ifdef VCT_PLATFORM_CTR
                dispNetwork2( pGlobal, vpos );
                drawflag = 1;
#else
                drawflag = 2;
#endif
            }
            else if ( pFw->IsTrigger(nn::pia::framework::BUTTON_DOWN))
            {
                vpos = s_Clip( (vpos + 1), 0, 1 );
#ifdef VCT_PLATFORM_CTR
                dispNetwork2( pGlobal, vpos );
                drawflag = 1;
#else
                drawflag = 2;
#endif
            }

            // Stop network check
            if ( netInfra->CheckConnection() == false )
            {
                if(stopflag == false)
                {
                    netInfra->Stop();
                }
                changemode = true;
                loopflag = false;
            }

#ifdef VCT_PLATFORM_CTR
            if(drawflag == 0)
            {
                pFw->EndFrameNoRender();
            }
            else
            {
                pFw->EndFrame();
            }
#else
            if(drawflag == 2)
            {
                dispNetwork2( pGlobal, vpos );
            }
            else if(drawflag == 3)
            {
                dispNetwork3( vpos );
            }
            pFw->EndFrame();
#endif
        }
        if(loopflag)
        {
            GameConference( pGlobal );
        }
    }
}


/*---------------------------------------------------------------------------*
  Name        : receiveCallback
  Description : net receive data callback function.
  Arguments   : srcCID  - src ConnectionID.
              : *buffer - received data.
              : size    - received data size.
  Returns     : -
 *---------------------------------------------------------------------------*/
static void receiveCallback( u32 srcCID, void *buffer, u32 size )
{
    VCT_HandleData( static_cast<u16>(ConvertID_Pia2VCT(srcCID & 0xFFFF)), (u8 *)buffer, size);
}


/*---------------------------------------------------------------------------*
  Name        : dispNetwork0
  Description : draw network start menu.
  Arguments   : *pGlobal - SampleVctGlobal variable.
              : vpos    - vertical position of cursor.
  Returns     : -
 *---------------------------------------------------------------------------*/
/*
    ----------------------------------------
    <<<<       Network Start Menu       >>>>
    ----------------------------------------
    > Network Match Mode : ANYBODY/FRIEND
      Game Mode          : 1000
      Max Station Num    : 8
      Start Network.
    ----------------------------------------
*/
static void dispNetwork0( SampleVctGlobal *pGlobal, u32 vpos )
{
    nn::pia::framework::Framework* pFw = nn::pia::framework::Framework::GetInstance();
    SampleVctNetParam       *param     = &pGlobal->netParameter;

    //framework->SetRenderTarget( NN_GX_DISPLAY0 );
    pFw->ChangeRenderTarget(true);
    pFw->SwapBuffers();

    //framework->SetRenderTarget( NN_GX_DISPLAY1 );
    pFw->ChangeRenderTarget(false);
    pFw->DrawText( 0, 8 *  4, "----------------------------------------" );
    pFw->DrawText( 0, 8 *  6, "<<<<       Network Start Menu       >>>>" );
    pFw->DrawText( 0, 8 *  8, "----------------------------------------" );
    if ( param->netMatchMode == SAMPLE_VCT_NET_MATCH_ANYBODY )
    {
        pFw->DrawText( 0, 8 * 10, "  Network Match Mode : ANYBODY" );
    }
    else
    {
        pFw->DrawText( 0, 8 * 10, "  Network Match Mode : FRIEND" );
    }
    pFw->DrawText( 0, 8 * 12, "  Game Mode          : %ld", param->gameMode );
    pFw->DrawText( 0, 8 * 14, "  Max Station Num    : %ld", param->maxConnect );
    pFw->DrawText( 0, 8 * 16, "  Start Network..." );
    pFw->DrawText( 0, 8 * (10 + 2 * vpos), ">" );
}


/*---------------------------------------------------------------------------*
  Name        : dispNetwork1
  Description : draw network connecting menu.
  Arguments   : vpos    - not use.
  Returns     : -
 *---------------------------------------------------------------------------*/
/*
    ----------------------------------------
    <<<<       Network Connecting       >>>>
    ----------------------------------------
*/
static void dispNetwork1( u32 vpos )
{
    (void)vpos;

    nn::pia::framework::Framework* pFw = nn::pia::framework::Framework::GetInstance();

    //framework->SetRenderTarget( NN_GX_DISPLAY0 );
    pFw->ChangeRenderTarget(true);
    pFw->SwapBuffers();

    //framework->SetRenderTarget( NN_GX_DISPLAY1 );
    pFw->ChangeRenderTarget(false);
    pFw->DrawText( 0, 8 *  4, "----------------------------------------" );
    pFw->DrawText( 0, 8 *  6, "<<<<       Network Connecting       >>>>" );
    pFw->DrawText( 0, 8 *  8, "----------------------------------------" );
}


/*---------------------------------------------------------------------------*
  Name        : dispNetwork2
  Description : draw network connected menu.
  Arguments   : *pGlobal - SampleVctGlobal variable.
              : vpos    - vertical position of cursor.
  Returns     : -
 *---------------------------------------------------------------------------*/
/*
    ----------------------------------------------------
    <<<<           Network Information...           >>>>
    ----------------------------------------------------
      HOST            : xxx.xxx.xxx.xxx
      MASK            : xxx.xxx.xxx.xxx
      DNS1            : xxx.xxx.xxx.xxx
      DNS2            : xxx.xxx.xxx.xxx
      GATEWAY         : xxx.xxx.xxx.xxx
      MAC             : xx:xx:xx:xx:xx:xx
      MY ConnectionID : xxx
      MY PID          : xxx
      OWNER PID       : xxx
    ----------------------------------------------------

    ----------------------------------------
    <<<<       Network Connected        >>>>
    ----------------------------------------
    > Conference Mode Menu.
      Stop Network.
    ----------------------------------------
*/
static void dispNetwork2( SampleVctGlobal *pGlobal, u32 vpos )
{
    nn::pia::framework::Framework* pFw = nn::pia::framework::Framework::GetInstance();
    NetInformationStructure   *netInfo   = &pGlobal->netInformation;
    nn::pia::common::String string;
    nn::pia::common::InetAddress inetAddress;

    //framework->SetRenderTarget( NN_GX_DISPLAY0 );
    pFw->ChangeRenderTarget(true);
    pFw->DrawText( 0, 8 *  0, "----------------------------------------------------" );
    pFw->DrawText( 0, 8 *  2, "<<<<           Network Information...           >>>>" );
    pFw->DrawText( 0, 8 *  4, "----------------------------------------------------" );
    netInfo->hostAddr.GetInetAddress(&inetAddress);
    inetAddress.GetAddressString(&string);
    pFw->DrawText( 0, 8 *  6, "  HOST            : %s", string.GetCharArray() );
    netInfo->netMask.GetInetAddress(&inetAddress);
    inetAddress.GetAddressString(&string);
    pFw->DrawText( 0, 8 *  8, "  MASK            : %s", string.GetCharArray() );
    netInfo->dns1.GetInetAddress(&inetAddress);
    inetAddress.GetAddressString(&string);
    pFw->DrawText( 0, 8 * 10, "  DNS1            : %s", string.GetCharArray() );
    netInfo->dns2.GetInetAddress(&inetAddress);
    inetAddress.GetAddressString(&string);
    pFw->DrawText( 0, 8 * 12, "  DNS2            : %s", string.GetCharArray() );
    netInfo->gateway.GetInetAddress(&inetAddress);
    inetAddress.GetAddressString(&string);
    pFw->DrawText( 0, 8 * 14, "  GATEWAY         : %s", string.GetCharArray() );
#ifdef VCT_PLATFORM_CTR
    pFw->DrawText( 0, 8 * 16, "  MAC             : %02X:%02X:%02X:%02X:%02X:%02X", netInfo->mac[0],
                                                                                   netInfo->mac[1],
                                                                                   netInfo->mac[2],
                                                                                   netInfo->mac[3],
                                                                                   netInfo->mac[4],
                                                                                   netInfo->mac[5] );
    pFw->DrawText( 0, 8 * 18, "  MY ConnectionID : %ld", netInfo->myCid );
    pFw->DrawText( 0, 8 * 20, "  MY PID          : %ld", netInfo->myPid );
    pFw->DrawText( 0, 8 * 22, "  OWNER PID       : %ld", netInfo->ownerPid );
    pFw->DrawText( 0, 8 * 28, "----------------------------------------------------" );
#else
    pFw->DrawText( 0, 8 * 16, "  MY ConnectionID : %ld", netInfo->myCid );
    pFw->DrawText( 0, 8 * 18, "  MY PID          : %ld", netInfo->myPid );
    pFw->DrawText( 0, 8 * 20, "  OWNER PID       : %ld", netInfo->ownerPid );
    pFw->DrawText( 0, 8 * 26, "----------------------------------------------------" );
#endif
    pFw->SwapBuffers();

    //framework->SetRenderTarget( NN_GX_DISPLAY1 );
    pFw->ChangeRenderTarget(false);
    pFw->DrawText( 0, 8 *  4, "----------------------------------------" );
    pFw->DrawText( 0, 8 *  6, "<<<<       Network Connected        >>>>" );
    pFw->DrawText( 0, 8 *  8, "----------------------------------------" );
    pFw->DrawText( 0, 8 * 10, "  Conference Mode Menu..." );
    pFw->DrawText( 0, 8 * 12, "  Stop Network..." );

    pFw->DrawText( 0, 8 * (10 + 2 * vpos), ">" );
}


/*---------------------------------------------------------------------------*
  Name        : dispNetwork3
  Description : draw network connecting menu.
  Arguments   : vpos    - not use.
  Returns     : -
 *---------------------------------------------------------------------------*/
/*
    ----------------------------------------
    <<<<     Network Disconnecting      >>>>
    ----------------------------------------
*/
static void dispNetwork3( u32 vpos )
{
    (void)vpos;

    nn::pia::framework::Framework* pFw = nn::pia::framework::Framework::GetInstance();

    //framework->SetRenderTarget( NN_GX_DISPLAY0 );
    pFw->ChangeRenderTarget(true);
    pFw->SwapBuffers();

    //framework->SetRenderTarget( NN_GX_DISPLAY1 );
    pFw->ChangeRenderTarget(false);
    pFw->DrawText( 0, 8 *  4, "----------------------------------------" );
    pFw->DrawText( 0, 8 *  6, "<<<<     Network Disconnecting      >>>>" );
    pFw->DrawText( 0, 8 *  8, "----------------------------------------" );
}
/* <<<< EOF >>>> */
