/*---------------------------------------------------------------------------*
  Project:  Voice Chat Sample.
  File:     vct_sample.cpp

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*
    ! Attention!
      This sample program makes use of the following files with some alterations.
       "SampleDemos/snd/SimplePlayer/wave.cpp",
       "SampleDemos/snd/SimplePlayer/wave.cppwave.h"
       
 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
    Include
 *---------------------------------------------------------------------------*/
#include "framework.h"
#include "vct_sample.h"
#include "game_network.h"
#ifdef VCT_PLATFORM_CTR
#include <nn/applet.h>
#else
#include <cafe/procui.h>
#endif
#include "net.h"
#include "utilities.h"


/*---------------------------------------------------------------------------*
    Anonymous namespace
 *---------------------------------------------------------------------------*/
namespace
{
// Application
    SndMicDevice              sSndMicDev;
    NetworkInfra              sNetInfra;
}

/*---------------------------------------------------------------------------*
    Function
 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
  Name        : SAMPLE_VCT_Initialize
  Description : initialize SampleVctGlobal.
  Arguments   : *pGlobal - SampleVctGlobal variable.
  Returns     : -
 *---------------------------------------------------------------------------*/
static void SAMPLE_VCT_Initialize( SampleVctGlobal *pGlobal )
{
    // Initialize system...
    pGlobal->system.sndmicDev         = NULL;
    pGlobal->system.netInfra          = NULL;

    // Initialize sequence...
    pGlobal->sequence                 = SAMPLE_VCT_SEQ_INIT;

    // Net parameter...
    pGlobal->netParameter.netMatchMode   = SAMPLE_VCT_NET_MATCH_ANYBODY;
    pGlobal->netParameter.gameMode       = SAMPLE_VCT_DEFAULT_GAME_MODE;
    pGlobal->netParameter.maxConnect     = SAMPLE_VCT_DEFAULT_MAX_CONNECT;
}

#ifdef VCT_PLATFORM_CTR
#else
//Variable for status of HOME Menu usage (0 = Foregound. 1 = Background. 2 = Moving from background to foregound).
u32 gHomeMenuStatus = 0;

//Release callback.
static u32 saveCallback(void* unused)
{
    OSSavesDone_ReadyToRelease();
    return 0;
}
//Request to move from background to foreground.
static u32 callbackAcquiredForeground(void* unused)
{
    gHomeMenuStatus = 2;
    return 0;
}
//Request to move from foreground to background.
static u32 callbackReleaseForeground(void* unused)
{
    gHomeMenuStatus = 1;
    return 0;
}
#endif

/*---------------------------------------------------------------------------*
  Name        : nnMain
  Description : Voice Chat sample main.
  Arguments   : -
  Returns     : -
 *---------------------------------------------------------------------------*/
#ifdef VCT_PLATFORM_CTR
extern "C" void nnMain( void )
#else
int main(int argc,char **argv)
#endif
{
    SampleVctGlobal vct_global;

#ifdef VCT_PLATFORM_CTR
    nn::applet::Enable();

    // Initialize os/fs...
    nn::fs::Initialize();
    const size_t ROMFS_BUFFER_SIZE = 1024 * 64;
    static char buffer[ROMFS_BUFFER_SIZE];
    NN_UTIL_PANIC_IF_FAILED( nn::fs::MountRom( "rom:", 16, 16, buffer, ROMFS_BUFFER_SIZE, true ) );

    // Initialize Alarm system...
    nn::os::InitializeAlarmSystem();
#else
    void* mem1Storage;
    u32 mem1Size;
    //Initialize the HOME Menu status variable.
    gHomeMenuStatus = 0;
    //Initialize ProcUI.
    ProcUIInitEx(&saveCallback, NULL);
    //Configure a region for storing and restoring MEM1.
    OSGetMemBound(OSMem_MEM1, (u32*)&mem1Storage, &mem1Size);
    mem1Storage = MEMAllocFromDefaultHeap(mem1Size);
    ProcUISetMEM1Storage(mem1Storage, mem1Size);
    //Set where to process system messages.
    ProcUIRegisterCallback(PROCUI_MESSAGE_ACQUIRE,callbackAcquiredForeground, NULL, 200);
    ProcUIRegisterCallback(PROCUI_MESSAGE_RELEASE,callbackReleaseForeground, NULL, 200);
    //USB Microphone
    sSndMicDev.InitializeUSBMIC();
#endif

    // Initialize pia common
    if(sNetInfra.Initialize()){
        // Initialize SampleVctGlobal...
        SAMPLE_VCT_Initialize( &vct_global );
        SAMPLE_PRINT("MemorySize : %d\n",nn::pia::common::GetMemoryUsage());
        
        // Copy system object...
        vct_global.system.sndmicDev = &sSndMicDev;
        vct_global.system.netInfra  = &sNetInfra;

        // Game Loop
        for(;;){
            if( vct_global.sequence == SAMPLE_VCT_SEQ_INIT )
            {
                if( NetworkStart( &vct_global ) < 0 )
                {
                    break;
                }
                vct_global.sequence = SAMPLE_VCT_SEQ_PROCESS;
            }
            else if( vct_global.sequence == SAMPLE_VCT_SEQ_PROCESS )
            {
                NetworkConnected( &vct_global );
                vct_global.sequence = SAMPLE_VCT_SEQ_INIT;
            }
        }
    }

    sNetInfra.Finalize();

    // Wait Infinity...
    while ( true )
    {
        // No Work...
    }
}

/* <<<< EOF >>>> */
